using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using XQUEST.Audio;
using XQUEST.GameObjectManagement;
using XQUEST.Helpers;
using XQUEST.Input;
using XQUEST.SpriteAnimation;

namespace AudioDemo
{
  /// <summary>
  /// This demo shows how to to play sound effects and music
  /// using the XQUEST AudioManager. It also shows how to adjust
  /// volume levels for individual categories, as well as setting
  /// a global volume level.
  /// </summary>
  public class AudioDemo : Game
  {
    private GraphicsDeviceManager graphics;
    private SpriteBatch spriteBatch;

    private AudioManager audioManager;

    private GameObjectManager gameObjectManager;
    private SpriteSheetInfo explosionSheet;

    private InputManager input;

    private SpriteFont font;

    public AudioDemo()
    {
      graphics = new GraphicsDeviceManager(this);
      Content.RootDirectory = "Content";
    }

    /// <summary>
    /// Allows the game to perform any initialization it needs to before starting to run.
    /// This is where it can query for any required services and load any non-graphic
    /// related content.  Calling base.Initialize will enumerate through any components
    /// and initialize them as well.
    /// </summary>
    protected override void Initialize()
    {
      // We create the AudioManager game component, passing in the
      // settings files created by XACT.
      audioManager =
        new AudioManager(this, Path.Combine(Content.RootDirectory, "AudioDemo.xgs"),
                         Path.Combine(Content.RootDirectory, "AudioDemoWaveBank.xwb"),
                         Path.Combine(Content.RootDirectory, "AudioDemoSoundBank.xsb"));

      // Specify the categories that we have created in XACT. (The "Music" category already
      // existed)
      audioManager.SpecifyCategories("Music", "Sound Effects");

      // Add it to the application framework.
      Components.Add(audioManager);

      // Upon start, we want to play the music, looped. We have 
      // already specified in XACT that we want this cue to loop, so all
      // we have to do here is play it.
      audioManager.PlayCue("Menu_Loop");

      // Use the game object manager to mange the explosions.
      gameObjectManager = new GameObjectManager(this);
      gameObjectManager.BlendMode = SpriteBlendMode.AlphaBlend;
      Components.Add(gameObjectManager);

      // Init explosion sheet.
      explosionSheet = new SpriteSheetInfo("explosionsheet", 64, 64, 4, 4);

      // Init input.
      input = new InputManager(this);
      Components.Add(input);

      base.Initialize();
    }

    /// <summary>
    /// LoadContent will be called once per game and is the place to load
    /// all of your content.
    /// </summary>
    protected override void LoadContent()
    {
      // Create a new SpriteBatch, which can be used to draw textures.
      spriteBatch = new SpriteBatch(GraphicsDevice);

      // Load a font for displaying text.
      font = Content.Load<SpriteFont>("Arial");
    }

    /// <summary>
    /// UnloadContent will be called once per game and is the place to unload
    /// all content.
    /// </summary>
    protected override void UnloadContent()
    {
      // TODO: Unload any non ContentManager content here
    }

    /// <summary>
    /// Allows the game to run logic such as updating the world,
    /// checking for collisions, gathering input, and playing audio.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Update(GameTime gameTime)
    {
      // Allows the game to exit
      if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
        Exit();

      ProcessInput();

      // Create explosions randomly. Each frame has a 2% chance of 
      // spawning an explosion.
      if (RandomHelper.RandomInt(0, 100) < 2)
      {
        // Try to recycle
        Explosion explosion =
          (Explosion) gameObjectManager.Recycle(typeof (Explosion));
        if (explosion == null)
          explosion =
            new Explosion(this, explosionSheet,
                          new Animation(explosionSheet.GetFrames(), 0.085f, false));

        // The explosion should have a random size and position.
        explosion.Scale = new Vector2(RandomHelper.RandomFloat(0.2f, 2.4f));
        explosion.Position =
          RandomHelper.RandomScreenPosition(GraphicsDevice.Viewport.Width - 100,
                                            GraphicsDevice.Viewport.Height - 100);

        // In the case where we recycled an explosion, we need to
        // reset animation flags.
        explosion.Sprite.Isvisible = true;
        explosion.CurrentAnimation.IsStarted = true;

        // Add explosion to the manager.
        gameObjectManager.Add(explosion);

        // Play explosion sound effect.
        audioManager.PlayCue("explosion1");
      }

      base.Update(gameTime);
    }

    private void ProcessInput()
    {
      // Adjust global volume with F1 and F2
      if (input.Keyboard.IsKeyDown(Keys.F1))
        audioManager.GlobalVolume += 0.4f;
      if (input.Keyboard.IsKeyDown(Keys.F2))
        audioManager.GlobalVolume -= 0.4f;

      // Adjust music volume with F5 and F6
      if (input.Keyboard.IsKeyDown(Keys.F5))
        audioManager.SetVolume("Music", audioManager.GetVolume("Music") + 0.8f);
      if (input.Keyboard.IsKeyDown(Keys.F6))
        audioManager.SetVolume("Music", audioManager.GetVolume("Music") - 0.8f);

      // Adjust sound effects volume with F9 and F10
      if (input.Keyboard.IsKeyDown(Keys.F9))
        audioManager.SetVolume("Sound Effects",
                               audioManager.GetVolume("Sound Effects") + 0.8f);
      if (input.Keyboard.IsKeyDown(Keys.F10))
        audioManager.SetVolume("Sound Effects",
                               audioManager.GetVolume("Sound Effects") - 0.8f);
    }

    /// <summary>
    /// This is called when the game should draw itself.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Draw(GameTime gameTime)
    {
      graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

      spriteBatch.Begin();

      // Draw information.
      spriteBatch.DrawString(font,
                             string.Format("Global volume: {0:0} [F1/F2 to adjust]",
                                           audioManager.GlobalVolume), Vector2.Zero,
                             Color.Black);

      spriteBatch.DrawString(font,
                             string.Format("Music volume: {0:0} [F5/F6 to adjust]",
                                           audioManager.GetVolume("Music")),
                             new Vector2(0, 40), Color.Black);

      spriteBatch.DrawString(font,
                             string.Format(
                               "Sound Effects volume: {0:0} [F9/F10 to adjust]",
                               audioManager.GetVolume("Sound Effects")),
                             new Vector2(0, 60), Color.Black);


      spriteBatch.End();
      base.Draw(gameTime);
    }
  }
}