using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using XQUEST.CameraSystem;
using XQUEST.GameObjectManagement;
using XQUEST.Helpers;
using XQUEST.Input;
using XQUEST.Misc;

namespace CameraDemo
{
  /// <summary>
  /// This is the main type for your game
  /// </summary>
  public class CameraDemo : Game
  {
    private GraphicsDeviceManager graphics;
    private SpriteBatch spriteBatch;

    // font for text
    private SpriteFont font;

    // this is the game's main character
    private Duck duck;

    // input and camera manager
    private InputManager input;
    private CameraManager cameraManager;

    // current camera Type
    private string currentCameraType;

    //cameras
    private FreeCamera freeCamera;
    private FirstPersonCamera firstPersonCamera;
    private FreeThirdPersonCamera freeThirdPersonCamera;
    private ThirdPersonCamera thirdPersonCamera;

    private Model boxModel;
    private Texture2D boxTexture;

    private GameObjectManager gameObjectManager;
    private SphereDrawer sphereDrawer;

    public CameraDemo()
    {
      graphics = new GraphicsDeviceManager(this);
      Content.RootDirectory = "Content";
    }

    public Model BoxModel
    {
      get { return boxModel; }
    }

    public Texture2D BoxTexture
    {
      get { return boxTexture; }
    }

    /// <summary>
    /// Allows the game to perform any initialization it needs to before starting to run.
    /// This is where it can query for any required services and load any non-graphic
    /// related content.  Calling base.Initialize will enumerate through any components
    /// and initialize them as well.
    /// </summary>
    protected override void Initialize()
    {
      // add game components
      input = new InputManager(this);
      Components.Add(input);

      cameraManager = new CameraManager(this);
      Components.Add(cameraManager);

      /// both the grid and the boxes are drawable gamecomponents,
      /// that just draw some static graphics on screen to use as reference
      /// when moving the the duck around.
      Grid3D grid = new Grid3D(this);
      grid.Width = 50;
      Components.Add(grid);

      /// initilialize cameras
      /// it's important to remember to call the camera's initialize methods,
      /// because without the aspect ratio and frustum won't be set up, leading
      /// to a faulty projection matrix.
      freeCamera = new FreeCamera(this);
      firstPersonCamera = new FirstPersonCamera(this);
      freeThirdPersonCamera = new FreeThirdPersonCamera(this);
      thirdPersonCamera = new ThirdPersonCamera(this);

      /// tweak the freecamera's movement speed so that it won't move too fast.
      /// and start a bit above and behind the duck
      freeCamera.Initialize();
      freeCamera.MovementSpeed = 0.3f;
      freeCamera.Position = new Vector3(0, 2, 2);

      /// tweak the nearPlane so that we won't see a bit of the ducks beak in the middle of the screen
      /// when in first person view.
      firstPersonCamera.Initialize();
      firstPersonCamera.NearPlane = 0.7f;

      thirdPersonCamera.Initialize();
      freeThirdPersonCamera.Initialize();

      // add the cameras to the camera manager.
      cameraManager.Add(thirdPersonCamera);
      cameraManager.Add(firstPersonCamera);
      cameraManager.Add(freeThirdPersonCamera);
      cameraManager.Add(freeCamera);

      gameObjectManager = new GameObjectManager(this);
      gameObjectManager.UseSpriteBatch = false;
      Components.Add(gameObjectManager);

      // set up the duck
      duck = new Duck(this, "Models//ducky_highres");
      duck.AddObserver(firstPersonCamera);
      duck.AddObserver(thirdPersonCamera);
      duck.AddObserver(freeThirdPersonCamera);
      gameObjectManager.Add(duck);

      base.Initialize();
    }

    /// <summary>
    /// LoadContent will be called once per game and is the place to load
    /// all of your content.
    /// </summary>
    protected override void LoadContent()
    {
      // Create a new SpriteBatch, which can be used to draw textures.
      spriteBatch = new SpriteBatch(GraphicsDevice);

      // load the arial font
      font = Content.Load<SpriteFont>("Arial");

      // set up the text helper
      TextHelper.Font = font;
      TextHelper.SpriteBatch = spriteBatch;

      // load the box model and texture.
      // saves memory by using static fields since we are going to create
      // 100 box instances.
      Box.boxModel = Content.Load<Model>("Models\\box");
      Box.boxTexture = Content.Load<Texture2D>("Textures\\boxtexture");

      for (int z = -5; z < 5; z++)
      {
        for (int x = -5; x < 5; x++)
        {
          Box box = new Box(this);
          box.Scale = 0.05f;
          box.Position = new Vector3(x*5 + 2, 0.5f, z*5 + 2);
          gameObjectManager.Add(box);
        }
      }
    }

    /// <summary>
    /// UnloadContent will be called once per game and is the place to unload
    /// all content.
    /// </summary>
    protected override void UnloadContent()
    {
      // TODO: Unload any non ContentManager content here
    }

    /// <summary>
    /// Allows the game to run logic such as updating the world,
    /// checking for collisions, gathering input, and playing audio.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Update(GameTime gameTime)
    {
      // Allows the game to exit
      if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
        Exit();

      gameObjectManager.ActiveCamera = cameraManager.ActiveCamera;

      // when we press F9 on keyboard, switch to next camera view.
      if (input.Keyboard.IsKeyPressed(Keys.F9))
      {
        cameraManager.ActivateNextCamera();
      }

      base.Update(gameTime);
    }

    /// <summary>
    /// This is called when the game should draw itself.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Draw(GameTime gameTime)
    {
      graphics.GraphicsDevice.Clear(ClearOptions.DepthBuffer | ClearOptions.Target,
                                    Color.CornflowerBlue, 1.0f, 0);

      base.Draw(gameTime);

      // draw help text for camera controls
      DrawInfo();
    }

    private void DrawInfo()
    {
      Camera currentCam = cameraManager.ActiveCamera;

      // print out information on screen
      if (currentCam is FreeCamera)
        currentCameraType = "Free Camera";
      else if (currentCam is FirstPersonCamera)
        currentCameraType = "First Person Camera";
      else if (currentCam is FreeThirdPersonCamera)
        currentCameraType = "Free Third Person Camera";
      else if (currentCam is ThirdPersonCamera)
        currentCameraType = "Third Person Camera";

      spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate,
                        SaveStateMode.SaveState);

      TextHelper.DrawShadowedText("Press [F9] to change camera", Vector2.Zero,
                                  Color.Black, Color.Pink);

      TextHelper.DrawShadowedText("-:" + currentCameraType + ":-", new Vector2(0, 20),
                                  Color.Black, Color.Pink);

      TextHelper.DrawShadowedText("Camera Controls:", new Vector2(0, 60),
                                  Color.Black, Color.Yellow);

      TextHelper.DrawShadowedText("Use WASD or Left Thumbstick to control the duck",
                                  new Vector2(300, 0), Color.Black, Color.White);

      switch (currentCameraType)
      {
        case "Free Camera":
          TextHelper.DrawShadowedText("[Arrow Keys, DPad]: Move Around",
                                      new Vector2(0, 80), Color.Black,
                                      Color.Yellow, new Vector2(0.75f));
          TextHelper.DrawShadowedText(
            "[ALT + Arrow Keys Up/Down, Left/Right Shoulder]: Move Up/Down",
            new Vector2(0, 100),
            Color.Black, Color.Yellow, new Vector2(0.75f));
          TextHelper.DrawShadowedText("[Mouse, Right Stick]: Look Around",
                                      new Vector2(0, 120), Color.Black,
                                      Color.Yellow, new Vector2(0.75f));
          TextHelper.DrawShadowedText(
            "[Mouse rightclick]: Move ahead in direction you are looking",
            new Vector2(0, 140), Color.Black,
            Color.Yellow, new Vector2(0.75f));
          break;
        case "First Person Camera":
          break;
        case "Free Third Person Camera":
          TextHelper.DrawShadowedText("[ALT + Arrows Up/Down]: Change Height",
                                      new Vector2(0, 80),
                                      Color.Black, Color.Yellow, new Vector2(0.75f));
          TextHelper.DrawShadowedText("[ALT + Arrows Left/Right]: Rotate",
                                      new Vector2(0, 100),
                                      Color.Black, Color.Yellow, new Vector2(0.75f));
          TextHelper.DrawShadowedText(
            "[Hold Right Mouse Button and Move Mouse]: Rotate and Zoom",
            new Vector2(0, 120),
            Color.Black, Color.Yellow, new Vector2(0.75f));
          break;

        case "Third Person Camera":
          TextHelper.DrawShadowedText("[ALT + Arrows Up/Down]: Change Height",
                                      new Vector2(0, 80), Color.Black,
                                      Color.Yellow, new Vector2(0.75f));
          TextHelper.DrawShadowedText(
            "[Hold Right Mouse Button and Move Mouse]: Zoom",
            new Vector2(0, 100), Color.Black,
            Color.Yellow, new Vector2(0.75f));
          TextHelper.DrawShadowedText("[Mouse Scroll Wheel]: Zoom",
                                      new Vector2(0, 120), Color.Black,
                                      Color.Yellow, new Vector2(0.75f));
          break;
      }

      spriteBatch.End();
    }
  }
}