using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Input;
using XQUEST.GameObjectManagement;
using XQUEST.Input;

namespace GameObjects_2D_Demo
{
  /// <summary>
  /// The cannon sits at the bottom of the screen and can
  /// shoot bullets at flying dragons.
  /// </summary>
  public class Cannon : SpriteGameObject
  {
    // The time it takes before the cannon can fire again.
    private float cooldownTime;

    // The elapsed time since the cannon was fired.
    private float timeSinceLastFire;

    public Cannon(Game game, string textureName, Rectangle? sourceRect)
      : base(game, textureName, sourceRect)
    {
      // We don't want to have the cannon collide with any other object,
      // so we set this flag to false to save processing time in the GameObjectManager.
      collideWithOtherObjects = false;

      // We don't want to recycle it either; it will just be one cannon and it will never die.
      recycle = false;

      // Can only fire every 0.25th second.
      cooldownTime = 0.25f;
    }

    public override void LoadContent(ContentManager content)
    {
      // This is where the sprite texture is being loaded. We can't
      // make any calculations or configurations of the sprite until
      // this has been done.
      base.LoadContent(content);

      // Set the origin of the sprite (the point it will rotate around)
      // to the bottom of it.
      sprite.Origin = new Vector2(Sprite.Width/2, Sprite.Height);

      // Set the position to bottom of screen, in the middle.
      Position.X = Game.GraphicsDevice.Viewport.Width/2;
      Position.Y = Game.GraphicsDevice.Viewport.Height;
    }

    public override void Update(GameTime gameTime, InputManager input)
    {
      // Increment time.
      timeSinceLastFire += (float) gameTime.ElapsedGameTime.TotalSeconds;

      // Rotate cannon based on keyboard input.
      if (input.Keyboard.IsKeyDown(Keys.Left))
        Sprite.Rotation -= 0.03f;
      if (input.Keyboard.IsKeyDown(Keys.Right))
        Sprite.Rotation += 0.03f;

      // Clamp rotation value in the range [-90, 90] degrees (we don't want cannon to point downwards).
      Sprite.Rotation =
        MathHelper.Clamp(Sprite.Rotation, -MathHelper.PiOver2, MathHelper.PiOver2);

      // Fire a bullet when the space key is down and the cannon is ready
      // for use.
      if (input.Keyboard.IsKeyDown(Keys.Space) && timeSinceLastFire >= cooldownTime)
      {
        // Try to recycle a bullet.
        Bullet bullet = (Bullet) gameObjectManager.Recycle(typeof (Bullet));

        // If no bullets are available for recycling, create a new one.
        if (bullet == null)
          bullet = new Bullet(Game, "Bullet", null);

        // Set the initial position of the bullet to the tip of the cannon,
        // using simple trigonometry.
        bullet.Position = Position;
        bullet.Position.X += (float) Math.Cos(MathHelper.PiOver2 - Sprite.Rotation)*
                             sprite.Height;
        bullet.Position.Y -= (float) Math.Sin(MathHelper.PiOver2 - Sprite.Rotation)*
                             sprite.Height;

        // Set the velocity (direction) based on the angle of the cannon.
        bullet.Velocity.X = (float) Math.Sin(Sprite.Rotation);
        bullet.Velocity.Y = (float) -Math.Cos(Sprite.Rotation);

        // Give it some speed.
        bullet.Velocity *= 9.0f; // new Vector2(9.0f);

        // Add it to the manager.
        gameObjectManager.Add(bullet);

        // Reset time.
        timeSinceLastFire = 0.0f;
      }

      base.Update(gameTime, input);
    }
  }
}