using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using XQUEST.Helpers;
using XQUEST.Input;
using XQUEST.SpriteAnimation;

// inputmanager is one of the helper classes

namespace InputDemo
{
  /// <summary>
  /// This demo shows how the input manager can be used to determine input to the game.
  /// </summary>
  public class InputDemo : Game
  {
    #region Fields

    private GraphicsDeviceManager graphics;
    private SpriteBatch spriteBatch;

    // AnimatedSprite object.
    private AnimatedSprite animatedSprite;

    // Sprite font used for drawing text.
    private SpriteFont font;

    // Stores the width and height of the screen.
    private int screenWidth, screenHeight;

    // stores the keys and buttons currently beeing held down
    private List<string> keysDown;
    private List<string> mouseDown;

    // stores the last key that was pressed
    private string lastKeyPressed;

    // stores the last mouse button that was pressed
    private string lastMouseButtonPressed;

    // the dragon's position on screen.
    private Vector2 dragonPosition;

    // if the dragon is moving, and the position to move to
    private bool isMoving;
    private Vector2 dragonMoveToPosition;

    // movement speed of the dragon
    private Vector2 speed;

    #endregion

    #region constructor

    public InputDemo()
    {
      graphics = new GraphicsDeviceManager(this);
      Content.RootDirectory = "Content";
    }

    #endregion

    #region initialization

    /// <summary>
    /// Allows the game to perform any initialization it needs to before starting to run.
    /// This is where it can query for any required services and load any non-graphic
    /// related content.  Calling base.Initialize will enumerate through any components
    /// and initialize them as well.
    /// </summary>
    protected override void Initialize()
    {
      // Get screen dimensions from the graphics device.
      screenWidth = GraphicsDevice.Viewport.Width;
      screenHeight = GraphicsDevice.Viewport.Height;

      //create a new instance of the input manager.
      InputManager input = new InputManager(this);
      //The input manager is a gameComponent and needs therefore to be added to the
      //components collection of the game.
      Components.Add(input);

      SpriteSheetInfo ssi = new SpriteSheetInfo("dragonsheet", 104, 52, 5, 1);

      Animation flyingAnimation =
        new Animation(ssi.GetFrames(), 0.1f, true, new int[] {0, 1, 2, 3, 4, 3, 2, 1});

      animatedSprite = new AnimatedSprite(ssi, flyingAnimation);
      animatedSprite.Origin = new Vector2(animatedSprite.Width/2, animatedSprite.Height/2);

      // get a default windows cursor in game
      IsMouseVisible = true;

      // initialize the list that holds the keys currently beeing held down
      keysDown = new List<string>();

      // initialize the list that holds the mouse buttons currently beeing held down
      mouseDown = new List<string>();

      // initialized dragon's position to middle of game window
      dragonPosition = new Vector2(screenWidth/2, screenHeight/2);
      dragonMoveToPosition = dragonPosition;

      // dragon is not moving at start, and speed is 2f each second;
      isMoving = false;
      speed = new Vector2(2f, 2f);
      base.Initialize();
    }

    #endregion

    #region content loading and unloading

    /// <summary>
    /// LoadContent will be called once per game and is the place to load
    /// all of your content.
    /// </summary>
    protected override void LoadContent()
    {
      // Create a new SpriteBatch, which can be used to draw textures.
      spriteBatch = new SpriteBatch(GraphicsDevice);

      // Load a font for displaying text.
      font = Content.Load<SpriteFont>("Arial");

      // set up the text helper
      TextHelper.Font = font;
      TextHelper.SpriteBatch = spriteBatch;

      // Load textures used by sprites.
      animatedSprite.LoadTexture(Content);
    }

    /// <summary>
    /// UnloadContent will be called once per game and is the place to unload
    /// all content.
    /// </summary>
    protected override void UnloadContent()
    {
      // TODO: Unload any non ContentManager content here
    }

    #endregion

    #region main update method

    /// <summary>
    /// Allows the game to run logic such as updating the world,
    /// checking for collisions, gathering input, and playing audio.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Update(GameTime gameTime)
    {
      // Allows the game to exit
      if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
        Exit();

      /// When you create a new instance of the InputManager it gets added to the
      /// game's services collection. You can always retrieve this instance
      /// with the following line:
      InputManager input = (InputManager) Services.GetService(typeof (InputManager));

      // Update the animation on the animated sprite.
      animatedSprite.UpdateAnimation(gameTime);

      // update the keyboard info
      UpdateKeyboardInfo(input);

      // update the mouse info
      UpdateMouseInfo(input);

      UpdateDragonPosition(input);

      base.Update(gameTime);
    }

    #endregion

    #region update dragon position, keyboard info and mouse info

    /// <summary>
    /// Update dragon position.
    /// Moves dragon based on which arrow keys are down, and where the last left-mouseclick was.
    /// </summary>
    /// <param name="input">the input manager</param>
    private void UpdateDragonPosition(InputManager input)
    {
      /// here we update the position of the dragon based on input
      /// we want to move the dragon with the arrow keys or the left thumbstick on the xbox gamepad
      /// we also want to position the dragon with left mouse click


      /// check which arrow keys or gamepad buttons are down and update position.
      /// Keep inside the game window.
      if (input.IsKeyOrButtonDown(Keys.Up, Buttons.LeftThumbstickUp))
      {
        dragonPosition.Y -= speed.Y;
        if (dragonPosition.Y < 0)
          dragonPosition.Y = 0;
      }
      if (input.IsKeyOrButtonDown(Keys.Down, Buttons.LeftThumbstickDown))
      {
        dragonPosition.Y += speed.Y;
        if (dragonPosition.Y > screenHeight)
          dragonPosition.Y = screenHeight;
      }
      if (input.IsKeyOrButtonDown(Keys.Left, Buttons.LeftThumbstickLeft))
      {
        dragonPosition.X -= speed.X;
        if (dragonPosition.X < 0)
          dragonPosition.X = 0;

        animatedSprite.Effects = SpriteEffects.None;
      }
      if (input.IsKeyOrButtonDown(Keys.Right, Buttons.LeftThumbstickRight))
      {
        dragonPosition.X += speed.X;
        if (dragonPosition.X > screenWidth)
          dragonPosition.X = screenWidth;

        animatedSprite.Effects = SpriteEffects.FlipHorizontally;
      }

      /// if left mousebutton is clicked, set the dragonMoveToPosition to the mouse position
      if (input.Mouse.IsLeftButtonPressed)
      {
        dragonMoveToPosition.X = input.Mouse.PositionX;
        dragonMoveToPosition.Y = input.Mouse.PositionY;
        isMoving = true;

        // flip dragon so that it's facing the way it's going
        if (dragonMoveToPosition.X > dragonPosition.X)
        {
          animatedSprite.Effects = SpriteEffects.FlipHorizontally;
        }
        else
        {
          animatedSprite.Effects = SpriteEffects.None;
        }
      }

      /// move the dragon closer to the target position if we are not there already
      if (dragonMoveToPosition != dragonPosition && isMoving)
      {
        ///find the vector between current positon and target position.
        ///this vector containts distance and direction to move now
        Vector2 direction = dragonMoveToPosition - dragonPosition;

        /// check to see if we would move the dragon too far,
        /// and if we would we need to set the dragon to target position.
        /// Otherwise the dragon would most likely never reach the target position,
        /// since the speed will take the dragon too far each time.
        if (direction.Length() < speed.Length())
        {
          dragonPosition = dragonMoveToPosition;
          isMoving = false;
        }
          /// if we are not close enough, we normalize the vector so that we only,
          /// keep the direction info and multiply this with the speed vector. 
          /// This new vector we add to the dragons's current position, and so the
          /// new position is a little bit closer to the target.
        else
        {
          direction.Normalize();
          dragonPosition += direction*speed;
        }
      }
    }

    /// <summary>
    /// Update the information about the state of the mouse
    /// </summary>
    /// <param name="input">the inputmanager</param>
    private void UpdateMouseInfo(InputManager input)
    {
      /// from input manager you can get the mouse state of up to 5
      /// mouse buttons. you can determine if a button was pressed,
      /// and if it is beeing held down or not. Here we check if
      /// the different mouse buttons was pressed ( clicked ).

      if (input.Mouse.IsLeftButtonPressed)
      {
        lastMouseButtonPressed = "left button";
      }
      if (input.Mouse.IsMiddleButtonPressed)
      {
        lastMouseButtonPressed = "middle button";
      }
      if (input.Mouse.IsRightButtonPressed)
      {
        lastMouseButtonPressed = "right button";
      }
      if (input.Mouse.IsXButton1Pressed)
      {
        lastMouseButtonPressed = "4th button";
      }
      if (input.Mouse.IsXButton2Pressed)
      {
        lastMouseButtonPressed = "5th button";
      }

      /// here we check if a mousebutton is beeing held down,
      /// and if it is, we add it to a list of mouse buttons
      /// beeing held down

      mouseDown.Clear();
      if (input.Mouse.IsLeftButtonDown)
      {
        mouseDown.Add("left button");
      }
      if (input.Mouse.IsMiddleButtonDown)
      {
        mouseDown.Add("middle button");
      }
      if (input.Mouse.IsRightButtonDown)
      {
        mouseDown.Add("right button");
      }
      if (input.Mouse.IsXButton1Down)
      {
        mouseDown.Add("4th button");
      }
      if (input.Mouse.IsXButton2Down)
      {
        mouseDown.Add("5th button");
      }
    }

    /// <summary>
    /// Update the information about the state of the keyboard
    /// </summary>
    /// <param name="input">the inputmanager</param>
    private void UpdateKeyboardInfo(InputManager input)
    {
      /// from the input manager we get the currentkeyboard state,
      /// and from the keyboard state we can get an array of keys
      /// currently beeing pressed down
      Keys[] keys = input.Keyboard.CurrentKeyboardStates[0].GetPressedKeys();

      // clear the keysDown list
      keysDown.Clear();

      // loop through the keys and add them to the keys down list
      foreach (Keys key in keys)
      {
        keysDown.Add(key.ToString());
        ///using inputmanager to check if the key was pressed
        ///not merely down, but down and then up again,
        ///without any keys beeing pressed in between.
        if (input.Keyboard.IsKeyPressed(key))
        {
          /// store the last key pressed, 
          /// so that it remains drawn until a new key is pressed
          lastKeyPressed = key.ToString();
        }
      }
    }

    #endregion

    #region Main Draw Method

    /// <summary>
    /// This is called when the game should draw itself.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Draw(GameTime gameTime)
    {
      /// When you create a new instance of the InputManager it gets added to the
      /// game's services collection. You can always retrieve this instance
      /// with the following line:
      InputManager input = (InputManager) Services.GetService(typeof (InputManager));

      graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

      spriteBatch.Begin();

      // draws info about keys on screen
      drawKeyInfo();
      // draws info about the mouse on screen
      drawMouseInfo(input);
      // draw dragon
      animatedSprite.Draw(spriteBatch, dragonPosition);

      spriteBatch.End();

      base.Draw(gameTime);
    }

    #endregion

    #region Draw key info and mouse info

    /// <summary>
    /// Draws info about which keys are beeing held down and pressed.
    /// Needs to be called withing spriteBatch.Begin() and spriteBatch.End()
    /// </summary>
    private void drawKeyInfo()
    {
      // draws keys beeing held down on screen
      TextHelper.DrawShadowedText("Keys held down:", Vector2.Zero, Color.Black, Color.Gray);
      int i = 15;
      foreach (string key in keysDown)
      {
        TextHelper.DrawShadowedText(key, new Vector2(0, i), Color.Black, Color.Gray);
        i = i + 15;
      }

      // draws last key pressed on screen
      TextHelper.DrawShadowedText("Key pressed:", new Vector2(150, 0), Color.Black, Color.Coral);
      if (lastKeyPressed != null)
      {
        TextHelper.DrawShadowedText(lastKeyPressed, new Vector2(150, 15), Color.Black, Color.Coral);
      }
    }

    /// <summary>
    /// Draws info about which mouse buttons are beeing held down and clicked and the cursor position.
    /// Needs to be called withing spriteBatch.Begin() and spriteBatch.End()
    /// </summary>
    private void drawMouseInfo(InputManager input)
    {
      // draws mouse buttons beeing held down on screen
      TextHelper.DrawShadowedText("mouse buttons held down:", new Vector2(320, 0), Color.Black, Color.Green);
      int i = 15;
      foreach (string button in mouseDown)
      {
        TextHelper.DrawShadowedText(button, new Vector2(320, i), Color.Black, Color.Green);
        i = i + 15;
      }

      // draws last mouse button pressed ( clicked ) on screen
      TextHelper.DrawShadowedText("mouse button pressed:", new Vector2(550, 0), Color.Black, Color.Pink);
      if (lastMouseButtonPressed != null)
      {
        TextHelper.DrawShadowedText(lastMouseButtonPressed, new Vector2(550, 15), Color.Black, Color.Pink);
      }

      //Draws the cursor position
      TextHelper.DrawShadowedText("Mouse position: " + input.Mouse.PositionX + ", " + input.Mouse.PositionY,
                                  new Vector2(screenWidth - 300, screenHeight - 30), Color.Black, Color.Yellow);
    }

    #endregion
  }
}