using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using XQUEST.CameraSystem;
using XQUEST.GameObjectManagement;
using XQUEST.Helpers;
using XQUEST.Input;

namespace Pointing_Demo
{
  /// <summary>
  /// This demo shows how to make game objects clickable with the mouse in game,
  /// by using XQUEST's Clickable Manager.
  /// </summary>
  public class PointingDemo : Game
  {
    #region Fields

    private GraphicsDeviceManager graphics;
    private SpriteBatch spriteBatch;

    // the 2 ducks to be drawn
    private Duck duck1;
    private Duck duck2;
    private Duck duck3;

    // the camera which enables us to actually see the ducks
    private FreeCamera camera;

    // some different managers that handles the camera, input, clicking and gameobjects.
    private CameraManager cameraManager;
    private InputManager input;
    private ClickableManager clickableManager;
    private GameObjectManager gameObjectManager;

    // font for text
    private SpriteFont font;

    #endregion

    #region Constructor

    public PointingDemo()
    {
      graphics = new GraphicsDeviceManager(this);
      Content.RootDirectory = "Content";
    }

    #endregion

    #region override methods

    /// <summary>
    /// Allows the game to perform any initialization it needs to before starting to run.
    /// This is where it can query for any required services and load any non-graphic
    /// related content.  Calling base.Initialize will enumerate through any components
    /// and initialize them as well.
    /// </summary>
    protected override void Initialize()
    {
      // set up the different managers and add them to components
      clickableManager = new ClickableManager(this);
      Components.Add(clickableManager);

      cameraManager = new CameraManager(this);
      Components.Add(cameraManager);

      input = new InputManager(this);
      Components.Add(input);

      gameObjectManager = new GameObjectManager(this);
      gameObjectManager.UseSpriteBatch = false;
      Components.Add(gameObjectManager);

      /// Grid3D is a drawable gamecomponent which is nice for debugging.
      /// Grid3D draws a grid on the XZ-plane, which is really good
      /// to have as a visual reference when looking at a scene.
      Components.Add(new Grid3D(this));

      // set up an instance of FreeCamera.
      camera = new FreeCamera(this);
      camera.Initialize();
      camera.MovementSpeed = 0.3f;
      camera.cameraLookOff = true;
      camera.Position = new Vector3(0, 3, 3);
      camera.Rotation = new Vector3(MathHelper.PiOver4, MathHelper.Pi, 0);


      // sets up the duck instances.
      duck1 = new Duck(this, "Models\\ducky_highres");
      duck1.Position = new Vector3(0, 0, 0);
      duck2 = new Duck(this, "Models\\ducky_highres");
      duck2.Position = new Vector3(1, 0, 0);
      duck3 = new Duck(this, "Models\\ducky_highres");
      duck3.Position = new Vector3(-1, 0, 0);

      /// here we add some objects to the appropriate manager.
      /// Duck goes to the GameObject manager, and camera goes
      /// to the Camera Manager.

      cameraManager.Add(camera);
      clickableManager.Add(duck1);
      clickableManager.Add(duck2);
      clickableManager.Add(duck3);
      gameObjectManager.Add(duck1);
      gameObjectManager.Add(duck2);
      gameObjectManager.Add(duck3);

      // we use a default windows cursor, so you can see where you are clicking.
      IsMouseVisible = true;

      base.Initialize();
    }


    /// <summary>
    /// LoadContent will be called once per game and is the place to load
    /// all of your content.
    /// </summary>
    protected override void LoadContent()
    {
      // Create a new SpriteBatch, which can be used to draw textures.
      spriteBatch = new SpriteBatch(GraphicsDevice);

      // load the arial font
      font = Content.Load<SpriteFont>("Arial");

      // set up the text helper
      TextHelper.Font = font;
      TextHelper.SpriteBatch = spriteBatch;
    }

    /// <summary>
    /// UnloadContent will be called once per game and is the place to unload
    /// all content.
    /// </summary>
    protected override void UnloadContent()
    {
      // TODO: Unload any non ContentManager content here
    }

    /// <summary>
    /// Allows the game to run logic such as updating the world,
    /// checking for collisions, gathering input, and playing audio.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Update(GameTime gameTime)
    {
      // Allows the game to exit
      if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
        Exit();

      /// need to tell the gameObjectManager which camera is the active camera.
      /// Supports hotswapping the cameras if you have several of them.
      gameObjectManager.ActiveCamera = cameraManager.ActiveCamera;

      base.Update(gameTime);
    }

    /// <summary>
    /// This is called when the game should draw itself.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Draw(GameTime gameTime)
    {
      graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

      spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate,
                        SaveStateMode.SaveState);

      // Draw info on screen
      TextHelper.DrawShadowedText("Duck #1 clicked: " + duck1.NClicked, Vector2.Zero,
                                  Color.Black, Color.Pink);
      TextHelper.DrawShadowedText("Duck #2 clicked: " + duck2.NClicked,
                                  new Vector2(0, 20), Color.Black, Color.Pink);
      TextHelper.DrawShadowedText("Duck #3 clicked: " + duck3.NClicked,
                                  new Vector2(0, 40), Color.Black, Color.Pink);

      TextHelper.DrawShadowedTextCentered("Rotate the ducks by clicking on them",
                                          new Vector2(400, 50), Color.Black,
                                          Color.Red);
      TextHelper.DrawShadowedTextCentered(
        "Leftclick: counter clockwise, rightclick: clockwise", new Vector2(400, 70),
        Color.Black, Color.Red);

      TextHelper.DrawShadowedText("Camera controls", new Vector2(550, 500),
                                  Color.Black, Color.Yellow);
      TextHelper.DrawShadowedText("[PgUp/Down]: Tilt", new Vector2(550, 520),
                                  Color.Black, Color.Yellow);
      TextHelper.DrawShadowedText("[Arrow keys]: Move", new Vector2(550, 540),
                                  Color.Black, Color.Yellow);
      TextHelper.DrawShadowedText("[Alt + Up/down]: Height", new Vector2(550, 560),
                                  Color.Black, Color.Yellow);

      spriteBatch.End();

      base.Draw(gameTime);
    }

    #endregion
  }
}