using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using XQUEST.SpriteAnimation;

namespace SpriteDemo
{
  /// <summary>
  /// This demo shows how to use both static and animated sprites
  /// in XQUEST.
  /// </summary>
  public class SpriteDemo : Game
  {
    private GraphicsDeviceManager graphics;
    private SpriteBatch spriteBatch;

    // Sprite and AnimatedSprite objects.
    private Sprite staticSprite;
    private AnimatedSprite animatedSprite;

    // Sprite font used for drawing text.
    private SpriteFont font;

    // Allow user to switch between showing static and animated sprite.
    private enum GameState
    {
      StaticSprite,
      AnimatedSprite
    };

    // Default to showing static sprite.
    private GameState gameState = GameState.StaticSprite;

    // Stores the width and height of the screen.
    private int screenWidth, screenHeight;

    public SpriteDemo()
    {
      graphics = new GraphicsDeviceManager(this);
      Content.RootDirectory = "Content";
    }

    /// <summary>
    /// Allows the game to perform any initialization it needs to before starting to run.
    /// This is where it can query for any required services and load any non-graphic
    /// related content.  Calling base.Initialize will enumerate through any components
    /// and initialize them as well.
    /// </summary>
    protected override void Initialize()
    {
      // Set up static sprite. Really simple.
      staticSprite = new Sprite("dragon");

      // Set up animated sprite using a sprite sheet and an animation object.
      // dragonsheet.png has 5 frames aligned on a single row, where each frame
      // is 104 pixels wide and 52 pixel high. 
      SpriteSheetInfo ssi = new SpriteSheetInfo("dragonsheet", 104, 52, 5, 1);

      // We need an animation object for describing how the frames will animate.
      // We want a delay of 0.1 seconds between each frame, and the frame sequence
      // will flip through the frames from the start to the end and back to the start
      // again.
      Animation flyingAnimation =
        new Animation(ssi.GetFrames(), 0.1f, true, new int[] {0, 1, 2, 3, 4, 3, 2, 1});

      // Finally, create the animated sprite.
      animatedSprite = new AnimatedSprite(ssi, flyingAnimation);

      base.Initialize();
    }

    /// <summary>
    /// LoadContent will be called once per game and is the place to load
    /// all of your content.
    /// </summary>
    protected override void LoadContent()
    {
      // Create a new SpriteBatch, which can be used to draw textures.
      spriteBatch = new SpriteBatch(GraphicsDevice);

      // Load a font for displaying text.
      font = Content.Load<SpriteFont>("Arial");

      // Load textures used by sprites.
      staticSprite.LoadTexture(Content);
      animatedSprite.LoadTexture(Content);

      // Get screen dimensions from the graphics device.
      screenWidth = GraphicsDevice.Viewport.Width;
      screenHeight = GraphicsDevice.Viewport.Height;
    }

    /// <summary>
    /// UnloadContent will be called once per game and is the place to unload
    /// all content.
    /// </summary>
    protected override void UnloadContent()
    {
      // TODO: Unload any non ContentManager content here
    }

    /// <summary>
    /// Allows the game to run logic such as updating the world,
    /// checking for collisions, gathering input, and playing audio.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Update(GameTime gameTime)
    {
      // Allows the game to exit
      if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
        Exit();

      // Check for keyboard input.
      if (Keyboard.GetState().IsKeyDown(Keys.F1))
        gameState = GameState.StaticSprite;
      if (Keyboard.GetState().IsKeyDown(Keys.F2))
        gameState = GameState.AnimatedSprite;

      // Update the animation on the animated sprite.
      animatedSprite.UpdateAnimation(gameTime);

      base.Update(gameTime);
    }

    /// <summary>
    /// This is called when the game should draw itself.
    /// </summary>
    /// <param name="gameTime">Provides a snapshot of timing values.</param>
    protected override void Draw(GameTime gameTime)
    {
      graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

      // We want the sprites to be centered on the screen.
      Vector2 position = new Vector2(screenWidth/2, screenHeight/2);

      spriteBatch.Begin();

      // Display instructions.
      spriteBatch.DrawString(font, "[F1] - Display static sprite.", Vector2.Zero,
                             gameState == GameState.StaticSprite
                               ? Color.Black
                               : Color.Gray);

      spriteBatch.DrawString(font, "[F2] - Display animated sprite.",
                             new Vector2(0, 20), gameState == GameState.AnimatedSprite
                               ? Color.Black
                               : Color.Gray);
      
      // Draw sprite based on game state.
      switch (gameState)
      {
        case GameState.StaticSprite:
          staticSprite.Draw(spriteBatch, position);
          break;

        case GameState.AnimatedSprite:
          animatedSprite.Draw(spriteBatch, position);
          break;
      }

      spriteBatch.End();

      base.Draw(gameTime);
    }
  }
}