using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework.Audio;

namespace XQUEST.Audio
{
  /// <summary>
  /// Used to internally manage audio categories.
  /// </summary>
  internal class AudioCategories
  {
    #region Fields

    // Dictionary holding categories like music, sound effect, ambient etc.
    private readonly Dictionary<string, AudioCategory> categories;
    
    // Dictionary holding volume levels of each category.
    private readonly Dictionary<string, float> categoryVolumes;

    // The default (global) volume level.
    public const float DefaultVolume = 100.0f;

    #endregion

    #region Constructors

    /// <summary>
    /// Creates a new AudioCategories.
    /// </summary>
    public AudioCategories()
    {
      categories = new Dictionary<string, AudioCategory>();
      categoryVolumes = new Dictionary<string, float>();
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// Adds a category.
    /// </summary>
    /// <param name="categoryName">The name of the category.</param>
    /// <param name="category">An AudioCategory object from the sound bank.</param>
    public void Add(string categoryName, AudioCategory category)
    {
      if (!categories.ContainsKey(categoryName))
      {
        // The audio engine operates on volume levels between 0.0 and 1.0 (or higher).
        // The XQUEST audio interface operates on volume levels between 0 and 100 (or higher).
        // Therefore, scale volume level.
        category.SetVolume(DefaultVolume / 100.0f);
        categories.Add(categoryName, category);
        categoryVolumes.Add(categoryName, DefaultVolume);
      }
    }

    /// <summary>
    /// Sets the volume of a category.
    /// </summary>
    /// <param name="categoryName">The name of the category.</param>
    /// <param name="volume">The volume level, between 0 (silence) and 100 (full)</param>
    public void SetVolume(string categoryName, float volume)
    {
      try
      {
        AudioCategory category = categories[categoryName];
          // throws exception if invalid category
        category.SetVolume(volume/100.0f);
        categoryVolumes[categoryName] = volume;
      }
      catch (Exception)
      {
        // Invalid category specified, just supress the error.
      }
    }

    /// <summary>
    /// Sets the global volume across all categories.
    /// </summary>
    /// <param name="volumeScale">The volume level to scale the category volumes by.</param>
    public void SetGlobalVolume(float volumeScale)
    {
      foreach (string categoryName in categories.Keys)
      {
        categories[categoryName].SetVolume(categoryVolumes[categoryName]/100.0f*volumeScale/
                                           100.0f);
      }
    }

    /// <summary>
    /// Stops playback of a category.
    /// </summary>
    /// <param name="categoryName">The category name</param>
    /// <param name="stopOptions">Stop options.</param>
    public void StopCategory(string categoryName, AudioStopOptions stopOptions)
    {
      if (categories.ContainsKey(categoryName))
        categories[categoryName].Stop(stopOptions);
    }

    /// <summary>
    /// Pauses playback of a category.
    /// </summary>
    /// <param name="categoryName">The category name</param>
    public void PauseCategory(string categoryName)
    {
      if (categories.ContainsKey(categoryName))
        categories[categoryName].Pause();
    }

    /// <summary>
    /// Resumes playback of a category.
    /// </summary>
    /// <param name="categoryName">The category name</param>
    public void ResumeCategory(string categoryName)
    {
      if (categories.ContainsKey(categoryName))
        categories[categoryName].Resume();
    }

    /// <summary>
    /// Gets the volume of a category.
    /// </summary>
    /// <param name="categoryName">The category name.</param>
    /// <returns>Current volume of the category.</returns>
    public float GetVolume(string categoryName)
    {
      if (categoryVolumes.ContainsKey(categoryName))
        return categoryVolumes[categoryName];
      else
        return -1.0f;
    }

    #endregion
  }
}