using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;

namespace XQUEST.Audio
{
  /// <summary>
  /// The AudioManager handles all your needs for playing and manipulating 
  /// sound and music.
  /// </summary>
  public class AudioManager : GameComponent
  {
    #region Fields

    // The AudioManager encapsulates the following 
    // XNA audio engine objects.
    private AudioEngine engine;
    private WaveBank waveBank;
    private SoundBank soundBank;

    // Categories of audio, defined in the XACT tool.
    private AudioCategories categories;

    // The global volume across all audio categories.
    private float globalVolume;

    // A list of the current active cues.
    private List<Cue> activeCues;

    #endregion

    #region Constructors

    /// <summary>
    /// Creates a new AudioManager.
    /// </summary>
    /// <param name="game">The Game to run under.</param>
    /// <param name="settingsFileName">Path to a global settings file.</param>
    /// <param name="nonStreamingWaveBankFilename">Path to the wave bank 
    /// file to load.</param>
    /// <param name="soundBankFilename">Path to the sound bank file to load.</param>
    public AudioManager(Game game, string settingsFileName,
                        string nonStreamingWaveBankFilename, string soundBankFilename) : base(game)
    {
      engine = new AudioEngine(settingsFileName);
      waveBank = new WaveBank(engine, nonStreamingWaveBankFilename);
      soundBank = new SoundBank(engine, soundBankFilename);

      activeCues = new List<Cue>();

      categories = new AudioCategories();

      // There will always be a category named default and music.
      // (These categories can't be deleted in XACT)
      categories.Add("Default", engine.GetCategory("Default"));
      categories.Add("Music", engine.GetCategory("Music"));

      GlobalVolume = 100.0f;

      // Add ourself as a service.
      game.Services.AddService(typeof (AudioManager), this);
    }

    #endregion

    #region Properties
    
    /// <summary>
    /// Gets the underlying XNA AudioEngine object.
    /// </summary>
    public AudioEngine AudioEngine
    {
      get { return engine; }
    }

    /// <summary>
    /// Gets the underlying XNA WaveBank object.
    /// </summary>
    public WaveBank WaveBank
    {
      get { return waveBank; }
    }

    /// <summary>
    /// Gets the underlying XNA SoundBank object.
    /// </summary>
    public SoundBank SoundBank
    {
      get { return soundBank; }
    }

    /// <summary>
    /// Gets or sets the global volume level (0-1000).
    /// </summary>
    public float GlobalVolume
    {
      get { return globalVolume; }
      set
      {
        globalVolume = MathHelper.Clamp(value, 0.0f, 1000.0f);
        categories.SetGlobalVolume(globalVolume);
      }
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// Use this method to specify categories that have been
    /// created in the XACT tool. This method should be called
    /// right after initialization of the audio manager. There is no
    /// need to specify categories that are automatically created
    /// in XACT, like "Default" and "Music".
    /// </summary>
    /// <param name="categoryNames"></param>
    public void SpecifyCategories(params string[] categoryNames)
    {
      foreach (string categoryName in categoryNames)
      {
        AudioCategory category = engine.GetCategory(categoryName);
        categories.Add(categoryName, category);
      }
    }

    /// <summary>
    /// Sets the volume of a category.
    /// </summary>
    /// <param name="category">The category whose volume should be set.</param>
    /// <param name="volume">The volume vale, between 0 (silence) and 100 (full).</param>
    public void SetVolume(string category, float volume)
    {
      categories.SetVolume(category, volume);
    }

    /// <summary>
    /// Gets the volume of a category.
    /// </summary>
    /// <param name="categoryName">The name of the category.</param>
    /// <returns>Current volume of the specified category.</returns>
    public float GetVolume(string categoryName)
    {
      return categories.GetVolume(categoryName);
    }

    /// <summary>
    /// Stops all sounds in a category.
    /// </summary>
    /// <param name="categoryName">The name of the category.</param>
    /// <param name="stopOptions">Stop options.</param>
    public void Stop(string categoryName, AudioStopOptions stopOptions)
    {
      categories.StopCategory(categoryName, stopOptions);
    }

    /// <summary>
    /// Pause all sounds in a category.
    /// </summary>
    /// <param name="categoryName">The name of the category.</param>
    public void Pause(string categoryName)
    {
      categories.PauseCategory(categoryName);
    }

    /// <summary>
    /// Resumes all paused sounds in a category.
    /// </summary>
    /// <param name="categoryName">The name of the category.</param>
    public void Resume(string categoryName)
    {
      categories.ResumeCategory(categoryName);
    }

    /// <summary>
    /// Gets a Cue object from the specified cue name.
    /// </summary>
    /// <param name="cueName">The name of the cue.</param>
    /// <returns>A Cue object representing the cue.</returns>
    public Cue GetCue(string cueName)
    {
      return soundBank.GetCue(cueName);
    }

    /// <summary>
    /// Plays a cue.
    /// </summary>
    /// <param name="cueName">Name of the cue as specified in the XACT tool.</param>
    public void PlayCue(string cueName)
    {
      Cue cue = soundBank.GetCue(cueName);
      cue.Play();
      activeCues.Add(cue);
    }

    /// <summary>
    /// Plays a cue.
    /// </summary>
    /// <param name="cue">A cue object to play.</param>
    public void PlayCue(Cue cue)
    {
      cue.Play();
      activeCues.Add(cue);
    }

    #endregion

    #region Overriden Interface

    /// <summary>
    /// Update the game.
    /// </summary>
    /// <param name="gameTime">The game time.</param>
    public override void Update(GameTime gameTime)
    {
      // Remove cues that have stopped.
      for (int i = activeCues.Count - 1; i >= 0; i--)
      {
        if (activeCues[i].IsStopped)
          activeCues.RemoveAt(i);
      }

      engine.Update();
    }

    #endregion
  }
}