using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.GameObjectManagement.BoundingVolumes;

namespace XQUEST.CameraSystem
{
  /// <summary>
  /// Provides a base for cameras. Override and implement
  /// camera functionality through the abstract View and Projection
  /// properties.
  /// </summary>
  public abstract class Camera : GameComponent
  {
    #region Fields

    // The split screen setting indicates the viewport of
    // the camera.
    protected CameraSplitScreenSetting cameraSplitScreenSetting;
    protected Viewport viewport;

    // The bounding frustum of the camera, 
    // can be used for frustum culling.
    protected XQBoundingFrustum boundingFrustum;

    #endregion

    #region Constructors

    protected Camera(Game game, CameraSplitScreenSetting cameraSplitScreenSetting)
      : base(game)
    {
      this.cameraSplitScreenSetting = cameraSplitScreenSetting;
      
    }

    #endregion

    #region Overriden Methods

    public override void Initialize()
    {
      boundingFrustum = new XQBoundingFrustum(View*Projection);
      CalculateViewport(Game.GraphicsDevice);

      base.Initialize();
    }

    public override void Update(GameTime gameTime)
    {
      // Calculate the frustum every frame based on
      // the view and projection matrices.
      boundingFrustum.BoundingFrustum.Matrix = View*Projection;

      base.Update(gameTime);
    }

    #endregion

    #region Properties

    /// <summary>
    /// Gets or sets the camers's split screen setting.
    /// </summary>
    public CameraSplitScreenSetting CameraSplitScreenSetting
    {
      get { return cameraSplitScreenSetting; }
      set
      {
        cameraSplitScreenSetting = value;
        CalculateViewport(Game.GraphicsDevice);
      }
    }

    /// <summary>
    /// Gets the current viewport constraining
    /// the camera.
    /// </summary>
    public Viewport Viewport
    {
      get { return viewport; }
    }

    /// <summary>
    /// Gets the bounding frustum of the camera.
    /// </summary>
    public XQBoundingFrustum BoundingFrustum
    {
      get { return boundingFrustum; }
    }

    #endregion

    #region Abstract Interface

    /// <summary>
    /// Gets the view matrix of this camera.
    /// </summary>
    public abstract Matrix View { get; }

    /// <summary>
    /// Gets the projection matrix of this camera.
    /// </summary>
    public abstract Matrix Projection { get; }

    #endregion

    #region Public Methods

    /// <summary>
    /// Sets the viewport based on the
    /// split screen setting.
    /// </summary>
    /// <param name="device">The graphics device.</param>
    public void CalculateViewport(GraphicsDevice device)
    {
      int width = device.PresentationParameters.BackBufferWidth;
      int height = device.PresentationParameters.BackBufferHeight;

      viewport = new Viewport();
      viewport.MinDepth = 0;
      viewport.MaxDepth = 1;

      switch (cameraSplitScreenSetting)
      {
        case CameraSplitScreenSetting.Fullscreen:
          viewport.X = 0;
          viewport.Y = 0;
          viewport.Width = width;
          viewport.Height = height;
          break;
        case CameraSplitScreenSetting.TopHalf:
          viewport.X = 0;
          viewport.Y = 0;
          viewport.Width = width;
          viewport.Height = height/2;
          break;
        case CameraSplitScreenSetting.BottomHalf:
          viewport.X = 0;
          viewport.Y = height/2;
          viewport.Width = width;
          viewport.Height = height/2;
          break;
        case CameraSplitScreenSetting.TopLeft:
          viewport.X = 0;
          viewport.Y = 0;
          viewport.Width = width/2;
          viewport.Height = height/2;
          break;
        case CameraSplitScreenSetting.TopRight:
          viewport.X = width/2;
          viewport.Y = 0;
          viewport.Width = width/2;
          viewport.Height = height/2;
          break;
        case CameraSplitScreenSetting.BottomLeft:
          viewport.X = 0;
          viewport.Y = height/2;
          viewport.Width = width/2;
          viewport.Height = height/2;
          break;
        case CameraSplitScreenSetting.BottomRight:
          viewport.X = width/2;
          viewport.Y = height/2;
          viewport.Width = width/2;
          viewport.Height = height/2;
          break;
      }
    }

    #endregion
  }
}