using System;
using Microsoft.Xna.Framework;

namespace XQUEST.CameraSystem
{
  /// <summary>
  /// Defines a 2D camera with this coordinate layout:
  /// 
  /// 
  /// ---------------------------------------
  /// |-width, -height       width, -height |
  /// |                                     |
  /// |                                     |
  /// |                                     |
  /// |               (0,0)                 |
  /// |                                     |
  /// |                                     |
  /// |                                     |
  /// | -width, height        width, height |
  /// ---------------------------------------
  /// where width and height are the width and height of the
  /// current viewport. Unlike the SpriteBatch coordinate system,
  /// Camera2D has its origin in the middle of the screen instead
  /// of the top-left corner to accomodate zooming/scaling. This means
  /// that to draw sprites using SpriteBatch, and still keep view and projection
  /// matrices of this camera valid, you need to subtract your sprite drawing positions by 
  /// half the viewport width and height:
  /// 
  /// Vector2 spritePos = ...
  /// spritePos = spritePos - new Vector2(camera2D.Viewport.Width/2, camera2D.Viewport.Width/2);
  /// , or call the method TransformSpriteBatchCoordinate(Vector2).
  ///  
  /// 
  /// </summary>
  public class Camera2D : Camera
  {
    #region Fields

    // The view matrix of the 2D camera is always the same. When we
    // move the camera, only the projection is updated.
    private static readonly Matrix view =
      Matrix.CreateLookAt(Vector3.UnitZ, -Vector3.UnitZ, Vector3.Up);

    // The projection matrix.
    private Matrix projection;

    // Position of the camera relative to the center of the screen.
    public Vector2 Position;

    // Zoom of the camera.
    private float zoom;

    // Rotation of the camera.
    private float rotation;

    // Indicates whether or not to move the camera relative to its
    // orientation.
    private bool isMovingUsingScreenAxis;

    #endregion

    #region Constructors

    /// <summary>
    /// Creates a new Camera2D.
    /// </summary>
    /// <param name="game">The Game to run under.</param>
    public Camera2D(Game game)
      : this(game, CameraSplitScreenSetting.Fullscreen)
    {
    }

    /// <summary>
    /// Creates a new Camera2D, specifying a split screen setting.
    /// </summary>
    /// <param name="game"></param>
    /// <param name="cameraSplitScreenSetting"></param>
    public Camera2D(Game game, CameraSplitScreenSetting cameraSplitScreenSetting)
      : base(game, cameraSplitScreenSetting)
    {
      Position = Vector2.Zero;
      zoom = 1.0f;
      rotation = 0.0f;
    }

    #endregion

    #region Properties

    /// <summary>
    /// Gets or sets a boolean value indicating whether or not to move the camera relative to its
    /// orientation.
    /// </summary>
    public bool IsMovingUsingScreenAxis
    {
      get { return isMovingUsingScreenAxis; }
      set { isMovingUsingScreenAxis = value; }
    }

    /// <summary>
    /// Gets or sets the rotation of the camera in radians.
    /// </summary>
    public float Rotation
    {
      get { return rotation; }
      set { rotation = value; }
    }

    /// <summary>
    /// Gets or sets the zoom level of the camera.
    /// </summary>
    public float Zoom
    {
      get { return zoom; }
      set { zoom = value; }
    }

    #endregion

    #region Overriden Interface

    /// <summary>
    /// Gets the view matrix of this camera.
    /// </summary>
    public override Matrix View
    {
      get { return view; }
    }

    /// <summary>
    /// Gets the projection matrix of this camera.
    /// </summary>
    public override Matrix Projection
    {
      get
      {
        projection = Matrix.CreateOrthographicOffCenter(
          (-viewport.Width/2)/zoom + Position.X,
          (viewport.Width/2)/zoom + Position.X,
          (viewport.Height/2)/zoom + Position.Y,
          (-viewport.Height/2)/zoom + Position.Y,
          .1f, 100.0f);

        return projection;
      }
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// Transforms coordinates used with SpriteBatch to coordinates
    /// in this camera coordinate system.
    /// </summary>
    /// <param name="spriteBatchCoordinate"></param>
    /// <returns></returns>
    public Vector2 TransformSpriteBatchCoordinate(Vector2 spriteBatchCoordinate)
    {
      return spriteBatchCoordinate - new Vector2(viewport.Width/2, viewport.Height/2);
    }

    /// <summary>
    /// Moves the camera right.
    /// </summary>
    /// <param name="distance">The distance to move.</param>
    public void MoveRight(ref float distance)
    {
      if (distance != 0)
      {
        if (isMovingUsingScreenAxis)
        {
          Position.X += (float) Math.Cos(-rotation)*distance;
          Position.Y += (float) Math.Sin(-rotation)*distance;
        }
        else
        {
          Position.X += distance;
        }
      }
    }

    /// <summary>
    /// Moves the camera left.
    /// </summary>
    /// <param name="distance">The distance to move.</param>
    public void MoveLeft(ref float distance)
    {
      if (distance != 0)
      {
        if (isMovingUsingScreenAxis)
        {
          Position.X -= (float) Math.Cos(-rotation)*distance;
          Position.Y -= (float) Math.Sin(-rotation)*distance;
        }
        else
        {
          Position.X -= distance;
        }
      }
    }

    /// <summary>
    /// Moves the camera up.
    /// </summary>
    /// <param name="distance">The distance to move.</param>
    public void MoveUp(ref float distance)
    {
      if (distance != 0)
      {
        if (isMovingUsingScreenAxis)
        {
          //using negative distance becuase 
          //"up" actually decreases the y value
          Position.X -= (float) Math.Sin(rotation)*distance;
          Position.Y -= (float) Math.Cos(rotation)*distance;
        }
        else
        {
          Position.Y -= distance;
        }
      }
    }

    /// <summary>
    /// Moves the camera down.
    /// </summary>
    /// <param name="distance">The distance to move.</param>
    public void MoveDown(ref float distance)
    {
      if (distance != 0)
      {
        if (isMovingUsingScreenAxis)
        {
          //using negative distance becuase 
          //"up" actually decreases the y value
          Position.X += (float) Math.Sin(rotation)*distance;
          Position.Y += (float) Math.Cos(rotation)*distance;
        }
        else
        {
          Position.Y += distance;
        }
      }
    }

    #endregion
  }
}