using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace XQUEST.CameraSystem
{
  /// <summary>
  /// Class for managing cameras.
  /// </summary>
  public class CameraManager : GameComponent
  {
    #region Fields

    private List<Camera> cameras;
    private int activeCameraIndex;

    #endregion

    #region Constructor

    /// <summary>
    /// Creates a new CameraManager.
    /// </summary>
    /// <param name="game"></param>
    public CameraManager(Game game)
      : base(game)
    {
      cameras = new List<Camera>();
      game.Services.AddService(typeof (CameraManager), this);
    }

    #endregion

    #region Properties

    /// <summary>
    /// Gets or sets the collection of cameras managed by
    /// the CameraManager.
    /// </summary>
    public List<Camera> Cameras
    {
      get { return cameras; }
      set { cameras = value; }
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// Adds a camera to the CameraManager.
    /// </summary>
    /// <param name="camera">The camera instance to add.</param>
    public void Add(Camera camera)
    {
      cameras.Add(camera);
      if (cameras.Count == 1)
      {
        activeCameraIndex = 0;
      }
      CheckIndex();
    }

    /// <summary>
    /// Removes a camera from the CameraManager.
    /// </summary>
    /// <param name="camera">The camera instance to remove.</param>
    public void Remove(Camera camera)
    {
      cameras.Remove(camera);
      CheckIndex();
    }

    /// <summary>
    /// Gets or sets the active camera.
    /// </summary>
    public Camera ActiveCamera
    {
      get { return cameras[activeCameraIndex]; }
      set
      {
        activeCameraIndex = cameras.IndexOf(value);
        CheckIndex();
      }
    }

    /// <summary>
    /// Actives the next camera.
    /// </summary>
    public void ActivateNextCamera()
    {
      activeCameraIndex = activeCameraIndex + 1%cameras.Count;
      CheckIndex();
    }

    #endregion

    #region Private Methods

    private bool IsIndexOutOfBounds()
    {
      return activeCameraIndex < 0 || activeCameraIndex >= cameras.Count;
    }

    private void CheckIndex()
    {
      if (cameras.Count == 0 || IsIndexOutOfBounds())
        activeCameraIndex = 0;
    }

    #endregion

    #region Overriden Interface

    /// <summary>
    /// Updates the CameraManager.
    /// </summary>
    /// <param name="gameTime">The game time.</param>
    public override void Update(GameTime gameTime)
    {
      if (cameras.Count != 0)
        cameras[activeCameraIndex].Update(gameTime);
    }

    #endregion
  }
}