using Microsoft.Xna.Framework;
using XQUEST.GameObjectManagement;
using XQUEST.Misc;

namespace XQUEST.CameraSystem
{
  /// <summary>
  /// A 3D camera observing a game object from a first person view.
  /// </summary>
  public class FirstPersonCamera : Camera3D, IObserver
  {
    #region Fields

    // Set the direction the camera points without rotation.
    private readonly Vector3 cameraReference = new Vector3(0, 0, 1);

    // Offset from the Position of the game object to its "head".
    private Vector3 headOffset;

    #endregion

    #region Constructors

    /// <summary>
    /// Creates a new FirstPersonCamera.
    /// </summary>
    /// <param name="game">The game to run under.</param>
    public FirstPersonCamera(Game game)
      : this(game, CameraSplitScreenSetting.Fullscreen)
    {
    }

    /// <summary>
    /// Creates a new FirstPersonCamera.
    /// </summary>
    /// <param name="game">The game to run under.</param>
    /// <param name="cameraSplitScreenSetting">
    /// A split screen setting indicating the portion of the 
    /// screen to direct the camera at.</param>
    public FirstPersonCamera(Game game,
                             CameraSplitScreenSetting cameraSplitScreenSetting)
      : base(game, cameraSplitScreenSetting)
    {
      // Default head offset value of 10 units above game object.
      headOffset = new Vector3(0, 10, 0);
    }

    #endregion

    #region Properties

    /// <summary>
    /// Gets or sets the head offset (the offset at which to place the
    /// camera relative to the observed game object).
    /// </summary>
    public Vector3 HeadOffset
    {
      get { return headOffset; }
      set { headOffset = value; }
    }

    #endregion

    #region Overridden Interface

    /// <summary>
    /// Notifies the camera of a change in the subject.
    /// </summary>
    /// <param name="someObject"></param>
    public void Notify(object someObject)
    {
      if (!(someObject is GameObject3D))
        return;

      GameObject3D subject = someObject as GameObject3D;

      // Head position almost at top of the object
      headOffset = new Vector3(0, subject.BoundingVolume.Height*0.90f, 0);

      // Create rotation matrix based on subject's current rotation.
      // Not interested in the rotation about the z-axis, since this is not
      // natural for a first person view.
      Matrix rotationMatrix = Matrix.CreateRotationY(subject.Rotation.Y);

      // Transform the head offset by the rotation matrix
      Vector3 transformedHeadOffset = Vector3.Transform(headOffset, rotationMatrix);

      // Set camera Position to the subject's "head".
      Position = subject.Position + transformedHeadOffset;

      // Transform the camera direction by the rotation matrix.
      Vector3 transformedReference =
        Vector3.Transform(cameraReference, rotationMatrix);

      // Calculate the position the camera is looking at.
      Vector3 cameraLookat = transformedReference + Position;

      // Create view matrix.
      viewMatrix = Matrix.CreateLookAt(Position, cameraLookat, Vector3.Up);

      // Create projection matrix
      projectionMatrix =
        Matrix.CreatePerspectiveFieldOfView(fieldOfView, viewport.AspectRatio,
                                            nearPlane,
                                            farPlane);
    }

    #endregion
  }
}