using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using XQUEST.Input;

namespace XQUEST.CameraSystem
{
  /// <summary>
  /// FreeCamera can move in any direction freely.
  /// </summary>
  public class FreeCamera : Camera3D
  {
    #region Fields

    private Vector3 rotation;
    private Vector3 movement;
    private float movementSpeed;

    private Vector2 mouseReference;

    public float gamePadSensitivity;
    public float mouseSensitivity;
    public bool invertLook;
    public bool cameraLookOff;

    #endregion

    #region Constructors

    /// <summary>
    /// Creates a new FreeCamera.
    /// </summary>
    /// <param name="game">The game to run under.</param>
    /// <param name="cameraSplitScreenSetting">
    /// A split screen setting indicating the portion of the 
    /// screen to direct the camera at.</param>
    public FreeCamera(Game game, CameraSplitScreenSetting cameraSplitScreenSetting)
      : base(game, cameraSplitScreenSetting)
    {
      rotation = Vector3.Zero;
      movement = Vector3.Zero;
      movementSpeed = 1.5f;

      gamePadSensitivity = 0.3f;
      mouseSensitivity = 0.02f;
      invertLook = false;
      cameraLookOff = false;
    }

    public FreeCamera(Game game) : this(game, CameraSplitScreenSetting.Fullscreen)
    {
    }

    #endregion

    #region Properties

    /// <summary>
    /// Gets or set the sensitivity of the game pad.
    /// </summary>
    public float GamePadSensitivity
    {
      get { return gamePadSensitivity; }
      set { gamePadSensitivity = value; }
    }

    /// <summary>
    /// Gets or sets the sensitivity of the mouse.
    /// </summary>
    public float MouseSensitivity
    {
      get { return mouseSensitivity; }
      set { mouseSensitivity = value; }
    }

    /// <summary>
    /// Gets or sets a boolean value indicating whether to invert the 
    /// mouse look.
    /// </summary>
    public bool InvertLook
    {
      get { return invertLook; }
      set { invertLook = value; }
    }

    /// <summary>
    /// Gets or sets the movement speed of the camera.
    /// </summary>
    public float MovementSpeed
    {
      get { return movementSpeed; }
      set { movementSpeed = value; }
    }

    /// <summary>
    /// Gets or sets the rotation of the camera.
    /// </summary>
    public Vector3 Rotation
    {
      get { return rotation; }
      set { rotation = value; }
    }

    #endregion

    #region Overridden Interface

    public override void Initialize()
    {
      base.Initialize();
      mouseReference = new Vector2(viewport.Width/2, viewport.Height/2);
    }

    public override void Update(GameTime gameTime)
    {
      base.Update(gameTime);

      HandleInput(gameTime);

      // Clamp the x rotation value.
      rotation.X =
        MathHelper.Clamp(rotation.X, -MathHelper.PiOver2 + 0.3f,
                         MathHelper.PiOver2 - 0.3f);

      // Create rotation matrix based on attached object's current rotation.
      // Not interested in the rotation about the z-axis, since this is not
      // natural for a first person view.
      Matrix rotationX = Matrix.CreateRotationX(rotation.X);
      Matrix rotationY = Matrix.CreateRotationY(rotation.Y);
      Matrix rotationMatrix = rotationX*rotationY;

      // Apply any movement. 
      Vector3 move = Vector3.Transform(movement, rotationY);
      Position += move;

      // Transform the camera direction by the rotation matrix.
      Vector3 transformedReference =
        Vector3.Transform(new Vector3(0, 0, 1), rotationMatrix);

      // Create view matrix.
      viewMatrix =
        Matrix.CreateLookAt(Position, Position + transformedReference, Vector3.Up);

      // Create projection matrix
      projectionMatrix =
        Matrix.CreatePerspectiveFieldOfView(fieldOfView, viewport.AspectRatio,
                                            nearPlane,
                                            farPlane);
    }

    #endregion

    #region Private Methods

    private void HandleInput(GameTime time)
    {
      InputManager input =
        (InputManager) Game.Services.GetService(typeof (InputManager));

      movement = Vector3.Zero;

      // Keyboard & game pad input
      if (input.IsKeyOrButtonDown(Keys.Up, Buttons.DPadUp) &&
          !input.Keyboard.IsKeyDown(Keys.LeftAlt))
      {
        movement.Z = movementSpeed;
      }
      if (input.IsKeyOrButtonDown(Keys.Down, Buttons.DPadDown) &&
          !input.Keyboard.IsKeyDown(Keys.LeftAlt))
      {
        movement.Z = -movementSpeed;
      }

      if ((input.Keyboard.IsKeyDown(Keys.LeftAlt) &&
           input.Keyboard.IsKeyDown(Keys.Up)) ||
          input.GamePad.IsButtonDown(Buttons.RightShoulder))
      {
        movement.Y = movementSpeed;
      }
      if ((input.Keyboard.IsKeyDown(Keys.LeftAlt) &&
           input.Keyboard.IsKeyDown(Keys.Down)) ||
          input.GamePad.IsButtonDown(Buttons.LeftShoulder))
      {
        movement.Y = -movementSpeed;
      }
      if (input.Keyboard.IsKeyDown(Keys.PageUp))
      {
        rotation.X -= movementSpeed*0.3f;
      }
      if (input.Keyboard.IsKeyDown(Keys.PageDown))
      {
        rotation.X += movementSpeed*0.3f;
      }
      if (input.Keyboard.IsKeyDown(Keys.Home))
      {
        rotation.X = 0;
      }
      if (input.IsKeyOrButtonDown(Keys.Left, Buttons.DPadLeft))
      {
        rotation.Y += movementSpeed*0.5f;
      }
      if (input.IsKeyOrButtonDown(Keys.Right, Buttons.DPadRight))
      {
        rotation.Y -= movementSpeed*0.5f;
      }

      Vector2 rightStickPosition = input.GamePad.GetRightStickPosition();
      rotation.X += -rightStickPosition.Y*gamePadSensitivity;
      rotation.Y += rightStickPosition.X*
                    (invertLook ? gamePadSensitivity : -gamePadSensitivity);
#if !XBOX
      // Handle mouse
      if (cameraLookOff != true)
      {
        if (input.Mouse.IsRightButtonDown)
        {
          Vector3 direction =
            new Vector3(0, -(float) Math.Sin(rotation.X),
                        (float) Math.Cos(rotation.X));
          movement = movementSpeed*direction;
        }
        
        MouseState currentMouseState = input.Mouse.CurrentMouseState;
        float deltaX = currentMouseState.X - mouseReference.X;
        float deltaY = currentMouseState.Y - mouseReference.Y;
        rotation.X += deltaY*mouseSensitivity;
        rotation.Y += deltaX*(invertLook ? mouseSensitivity : -mouseSensitivity);

        Mouse.SetPosition(viewport.Width/2, viewport.Height/2);
      }

#endif
    }

    #endregion
  }
}