using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using XQUEST.GameObjectManagement;
using XQUEST.Input;
using XQUEST.Misc;

namespace XQUEST.CameraSystem
{
  /// <summary>
  /// Camera follows a subject 
  /// </summary>
  public class ThirdPersonCamera : Camera3D, IObserver
  {
    #region Fields

    // Camera control variables.
    protected float cameraYaw;
    protected float heightChangeSpeed;
    protected float zoomChangeSpeed;
    protected float maxCameraPositionY;
    protected float minCameraPositionY;
    protected float maxCameraPositionZ;
    protected float minCameraPositionZ;

    // HOw much the camera has moved from its default position.
    protected float deltaY, deltaZ;

    // The default camera position.
    protected Vector3 defaultCameraPosition;

    #endregion

    #region Properties

    /// <summary>
    /// Gets or sets the speed of zooming in our out on the game object
    /// </summary>
    public float ZoomChangeSpeed
    {
      get { return zoomChangeSpeed; }
      set { zoomChangeSpeed = value; }
    }

    /// <summary>
    /// Gets or sets the speed of moving the camera up or down
    /// </summary>
    public float HeightChangeSpeed
    {
      get { return heightChangeSpeed; }
      set { heightChangeSpeed = value; }
    }

    /// <summary>
    /// Gets or sets the current rotation of the camera around the game object, in radians
    /// </summary>
    public float CameraYaw
    {
      get { return cameraYaw; }
      set { cameraYaw = value; }
    }

    /// <summary>
    /// Gets or sets the camera's minimum allowed horizontal distance from the game object
    /// </summary>
    public float MinCameraPositionZ
    {
      get { return minCameraPositionZ; }
      set { minCameraPositionZ = value; }
    }

    /// <summary>
    /// Gets or sets the camera's maxium allowed horizontal distance from the game object
    /// </summary>
    public float MaxCameraPositionZ
    {
      get { return maxCameraPositionZ; }
      set { maxCameraPositionZ = value; }
    }

    /// <summary>
    /// Gets or sets the camera's minimum allowed vertical distance from the game object
    /// </summary>
    public float MinCameraPositionY
    {
      get { return minCameraPositionY; }
      set { minCameraPositionY = value; }
    }

    /// <summary>
    /// Gets or sets the camera's maximum allowed vertical distance from the game object
    /// </summary>
    public float MaxCameraPositionY
    {
      get { return maxCameraPositionY; }
      set { maxCameraPositionY = value; }
    }

    #endregion

    #region Constructors

    public ThirdPersonCamera(Game game,
                             CameraSplitScreenSetting cameraSplitScreenSetting)
      : base(game, cameraSplitScreenSetting)
    {
    }

    public ThirdPersonCamera(Game game)
      : base(game)
    {
    }

    #endregion

    #region Methods

    /// <summary>
    /// Updates input from keyboard and mouse.
    /// </summary>
    protected virtual void UpdateCameraInput()
    {
      InputManager input =
        (InputManager) Game.Services.GetService(typeof (InputManager));

      // KEYBOARD INPUT
      if (input.Keyboard.IsKeyDown(Keys.LeftAlt) &&
          input.Keyboard.IsKeyDown(Keys.Down))
      {
        deltaY -= heightChangeSpeed;
      }
      else if (input.Keyboard.IsKeyDown(Keys.LeftAlt) &&
               input.Keyboard.IsKeyDown(Keys.Up))
      {
        deltaY += heightChangeSpeed;
      }


      // MOUSE INPUT
#if !XBOX
      if (input.Mouse.IsRightButtonDown)
      {
        if (input.Mouse.PositionDelta.Y < 0)
        {
          deltaY -= heightChangeSpeed;
        }
        else if (input.Mouse.PositionDelta.Y > 0)
        {
          deltaY += heightChangeSpeed;
        }
      }
      if (input.Mouse.ScrollWheelDelta != 0)
      {
        if (input.Mouse.ScrollWheelDelta < 0)
        {
          deltaZ += zoomChangeSpeed;
        }
        else if (input.Mouse.ScrollWheelDelta > 0)
        {
          deltaZ -= zoomChangeSpeed;
        }
      }
#endif

      deltaY = MathHelper.Clamp(deltaY, minCameraPositionY - defaultCameraPosition.Y,
                                maxCameraPositionY - defaultCameraPosition.Y);
      deltaZ = MathHelper.Clamp(deltaZ, minCameraPositionZ - defaultCameraPosition.Z,
                                maxCameraPositionZ - defaultCameraPosition.Z);
    }

    #endregion

    #region Overridden Interface

    public override void Update(GameTime gameTime)
    {
      UpdateCameraInput();
      base.Update(gameTime);
    }

    public void Notify(object someObject)
    {
      if (!(someObject is GameObject3D))
        return;

      GameObject3D subject = someObject as GameObject3D;

      Vector3 size =
        new Vector3(subject.BoundingVolume.Width, subject.BoundingVolume.Height,
                    subject.BoundingVolume.Depth);

      // Camera should be a little behind and a litte up from the subject.
      defaultCameraPosition.Z = 3*size.Z;
      defaultCameraPosition.Y = 2*size.Y;

      Position.Z = defaultCameraPosition.Z + deltaZ;
      Position.Y = defaultCameraPosition.Y + deltaY;

      heightChangeSpeed = 0.1f*size.Y;
      zoomChangeSpeed = 0.1f*size.Z;

      maxCameraPositionY = 5*size.Y;
      minCameraPositionY = 0;
      maxCameraPositionZ = 5*size.Z;
      minCameraPositionZ = size.Z;

      // Camera's position behind game object
      Vector3 thirdPersonReference = new Vector3(0, Position.Y, -Position.Z);

      // Camera should be rotated along with the subject.
      Matrix rotationMatrix =
        Matrix.CreateRotationY(this is FreeThirdPersonCamera
                                 ? cameraYaw
                                 : subject.Rotation.Y);

      // Create a vector pointing the direction the camera is facing.
      Vector3 transformedReference =
        Vector3.Transform(thirdPersonReference, rotationMatrix);

      // Calculate the position the camera is looking from.
      Vector3 cameraPosition = transformedReference + subject.Position;

      // Set up the view matrix
      viewMatrix =
        Matrix.CreateLookAt(cameraPosition, subject.Position,
                            new Vector3(0.0f, 1.0f, 0.0f));

      // Set up the projection matrix
      projectionMatrix =
        Matrix.CreatePerspectiveFieldOfView(fieldOfView, viewport.AspectRatio,
                                            NearPlane, farPlane);
    }

    #endregion
  }
}