using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.GameObjectManagement.BoundingVolumes;
using XQUEST.Input;
using XQUEST.Misc;

namespace XQUEST.GameObjectManagement
{
  public abstract class GameObject : IObservable
  {
    #region Fields

    protected Game game;

    protected GameObjectManager gameObjectManager;

    protected bool isInitialized;
    protected bool isVisible;

    // The state of the game object.
    protected GameObjectState state;

    // Boolean flag indicating whether the game object should be
    // recycled after its death.
    protected bool recycle;

    // Flag indicating wheter this game object can collide with 
    // other game objects.
    protected bool collideWithOtherObjects;

    // View and projection matrices for the game object.
    private Matrix view;
    private Matrix projection;

    // List of observers of this game object (typically cameras).
    private List<IObserver> observers;

    #endregion

    #region Constructors

    protected GameObject(Game game)
    {
      this.game = game;
      isInitialized = false;
      isVisible = true;

      state = GameObjectState.Alive;
      recycle = false;

      collideWithOtherObjects = true;

      view = Matrix.Identity;
      projection = Matrix.Identity;

      observers = new List<IObserver>();
    }

    #endregion

    #region Properties

    /// <summary>
    /// The Game instance.
    /// </summary>
    public Game Game
    {
      get { return game; }
      set { game = value; }
    }

    public bool IsInitialized
    {
      get { return isInitialized; }
      set { isInitialized = value; }
    }

    public bool IsVisible
    {
      get { return isVisible; }
      set { isVisible = value; }
    }
    
    public GameObjectState State
    {
      get { return state; }
      set { state = value; }
    }

    public bool Recycle
    {
      get { return recycle; }
      set { recycle = value; }
    }
    
    public bool CollideWithOtherObjects
    {
      get { return collideWithOtherObjects; }
      set { collideWithOtherObjects = value; }
    }

    public Matrix View
    {
      get { return view; }
      set { view = value; }
    }

    public Matrix Projection
    {
      get { return projection; }
      set { projection = value; }
    }

    #endregion

    #region Abstract and Virtual Methods

    // Defines the bounding volume of the game object.
    public abstract IBoundingVolume BoundingVolume
    {
      get;
    }
    
    // Collison detection interface. CollidesWith should define how
    // game objects can collide and return whether or not they do.
    // For example, GameObject2D defines a collision to have ocurred
    // between two GameObject2Ds if their bounding rectangles intersect.
    // Normally, you wouldn't implement CollidesWith in your own game objects,
    // unless you'd want more control over how the actucal checking is done (for
    // performance reasons).
    public virtual bool CollidesWith(GameObject gameObject)
    {
      return BoundingVolume.Intersects(gameObject.BoundingVolume);
    }

    // CollisionOccured is the callback method which is called in
    // case the game object was involved in a collision.
    public virtual void CollisionOccured(GameObject collisionObject){}

    public virtual void Initialize()
    {
      // Get the game object manager service.
      gameObjectManager =
        (GameObjectManager) Game.Services.GetService(typeof (GameObjectManager));
    }
    
    public virtual void LoadContent(ContentManager content){}
    public virtual void Update(GameTime gameTime, InputManager input)
    {
      if (input == null)
        throw new Exception("An InputManager must be set up and added to Game.Components!");

      // Notify all observers this object has been changed.
      foreach (IObserver observer in observers)
      {
        observer.Notify(this);
      }
    }
    public virtual void Draw(GameTime gameTime, SpriteBatch spriteBatch){ }

    #endregion

    #region IObservable Implementation

    public void AddObserver(IObserver observer)
    {
      observers.Add(observer);
    }

    public void RemoveObserver(IObserver observer)
    {
      observers.Remove(observer);
    }

    #endregion
  }
}
