using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.CameraSystem;
using XQUEST.Input;

namespace XQUEST.GameObjectManagement
{
  /// <summary>
  /// The GameObjectManager manages a list of GameObjects
  /// </summary>
  public class GameObjectManager : DrawableGameComponent
  {
    #region Fields

    // Maximum number of game objects the recycle list can hold.
    private const int MaxRecycleCount = 100;

    // List of active game objects.
    private readonly LinkedList<GameObject> activeGameObjects;

    // List of dead game objects. A game object with its recycle flag
    // set to true will end up here after it death.
    private readonly LinkedList<GameObject> recycleList;

    private readonly LinkedList<GameObject> newlyAdded;
    private readonly LinkedList<GameObject> newlyDeleted;

    private readonly List<GameObject> collisionObjects;

    // The input manager.
    private InputManager input;

    // The active camera;
    private Camera activeCamera;

    // A sprite batch.
    private SpriteBatch spriteBatch;

    // Are we done with initialization of GameObjectManager (Initialize + LoadContent)
    private bool isInitialized;

    // Should collision detection be handled?
    private bool isCollisionDetection;

    // SpriteBatch control variables.
    private SpriteBlendMode blendMode = SpriteBlendMode.None;
    private SpriteSortMode sortMode = SpriteSortMode.Immediate;
    private SaveStateMode stateMode = SaveStateMode.SaveState;
    private Matrix transformMatrix = Matrix.Identity;

    private bool useSpriteBatch = true;

    // How many objects are drawn per frame?
    private int drawCount;

    #endregion

    #region Constructors

    /// <summary>
    /// Creates a new GameObjectManager.
    /// </summary>
    /// <param name="game">The Game instance.</param>
    public GameObjectManager(Game game)
      : base(game)
    {
      activeGameObjects = new LinkedList<GameObject>();
      recycleList = new LinkedList<GameObject>();
      newlyAdded = new LinkedList<GameObject>();
      newlyDeleted = new LinkedList<GameObject>();
      collisionObjects = new List<GameObject>();

      // Add the manager as a service
      game.Services.AddService(typeof (GameObjectManager), this);

      isInitialized = false;
      isCollisionDetection = true;
    }

    #endregion

    #region Overriden Interface

    public override void Initialize()
    {
      input = (InputManager) Game.Services.GetService(typeof (InputManager));
      if (input == null)
        input = new InputManager(Game);

      foreach (GameObject gameObject in newlyAdded)
      {
        if (!gameObject.IsInitialized)
          gameObject.Initialize();
      }

      base.Initialize();
    }

    protected override void LoadContent()
    {
      // Create SpriteBatch
      spriteBatch = new SpriteBatch(GraphicsDevice);

      foreach (GameObject gameObject in newlyAdded)
      {
        if (!gameObject.IsInitialized)
        {
          gameObject.LoadContent(Game.Content);
          gameObject.IsInitialized = true;
        }
      }

      isInitialized = true;
      base.LoadContent();
    }

    public override void Draw(GameTime gameTime)
    {
      if (useSpriteBatch)
        spriteBatch.Begin(blendMode, sortMode, stateMode, transformMatrix);

      drawCount = 0;

      foreach (GameObject gameObject in activeGameObjects)
      {
        if (gameObject.IsVisible)
        {
          // Do frustum culling if a camera is active.
          if (activeCamera == null ||
              activeCamera.BoundingFrustum.Contains(gameObject.BoundingVolume) !=
              ContainmentType.Disjoint)
          {
            gameObject.Draw(gameTime, spriteBatch);
            drawCount++;
          }
        }
      }

      if (useSpriteBatch)
        spriteBatch.End();

      base.Draw(gameTime);
    }

    public override void Update(GameTime gameTime)
    {
      foreach (GameObject gameObject in activeGameObjects)
      {
        switch (gameObject.State)
        {
          case GameObjectState.Alive:
          case GameObjectState.Dying:
            gameObject.Update(gameTime, input);

            // Update the game object's view and projection matrices.
            if (activeCamera != null)
            {
              gameObject.View = activeCamera.View;
              gameObject.Projection = activeCamera.Projection;
            }

            if (isCollisionDetection && gameObject.CollideWithOtherObjects)
              collisionObjects.Add(gameObject);
            break;
          case GameObjectState.Dead:
            newlyDeleted.AddLast(gameObject);
            if (gameObject.Recycle)
            {
              // Maintain capacity.
              while (recycleList.Count >= MaxRecycleCount)
                recycleList.RemoveFirst(); // Remove oldest.

              // Add to recycle list.
              recycleList.AddLast(gameObject);
            }
            break;
        }
      }

      // Remove dead game objects.
      foreach (GameObject gameObject in newlyDeleted)
        activeGameObjects.Remove(gameObject);

      // Add newly added game objects.
      foreach (GameObject gameObject in newlyAdded)
        activeGameObjects.AddLast(gameObject);

      DetectCollisions();

      newlyDeleted.Clear();
      newlyAdded.Clear();
      collisionObjects.Clear();

      base.Update(gameTime);
    }

    private void DetectCollisions()
    {
      for (int i = 0; i < collisionObjects.Count; i++)
      {
        GameObject go1 = collisionObjects[i];
        for (int j = i + 1; j < collisionObjects.Count; j++)
        {
          GameObject go2 = collisionObjects[j];
          if (go1.CollidesWith(go2))
          {
            go1.CollisionOccured(go2);
            go2.CollisionOccured(go1);
          }
        }
      }
    }

    #endregion

    #region Private Methods

    private void OnNewGameObject(GameObject gameObject)
    {
      if (isInitialized && !gameObject.IsInitialized)
      {
        // We are adding the game object after the initialization
        // of the GameObjectManager, so we manually call the Initialize 
        // and LoadContent methods of the game object. Adding game objects
        // to the GameObjectManager during the game loop is generally a 
        // bad idea - use recycle lists instead!
        gameObject.Initialize();
        gameObject.LoadContent(Game.Content);
        gameObject.IsInitialized = true;
      }
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// Recycles a dead game object.
    /// </summary>
    /// <param name="type">The type of the game object to recycle.</param>
    /// <returns>The recycled game object, or null if no game object was found to recycle.</returns>
    public GameObject Recycle(Type type)
    {
      // Find the first game object of type type.
      GameObject gameObject = null;
      foreach (GameObject go in recycleList)
      {
        if (go.GetType() == type)
        {
          gameObject = go;
          break;
        }
      }

      // If we found one, remove it from the recycle list
      // and set its state to Alive.
      if (gameObject != null)
      {
        recycleList.Remove(gameObject);

        // It's Aliiiiiiiiiive.
        gameObject.State = GameObjectState.Alive;
      }

      return gameObject;
    }

    #endregion

    #region Delegated Members

    public void Add(GameObject item)
    {
      if (!activeGameObjects.Contains(item))
      {
        newlyAdded.AddLast(item);
        OnNewGameObject(item);
      }
    }

    public void Clear()
    {
      newlyAdded.Clear();
      newlyDeleted.Clear();
      activeGameObjects.Clear();
      recycleList.Clear();
    }

    #endregion

    #region Properties

    public bool UseSpriteBatch
    {
      get { return useSpriteBatch; }
      set { useSpriteBatch = value; }
    }

    public LinkedList<GameObject> ActiveGameObjects
    {
      get { return activeGameObjects; }
    }

    public LinkedList<GameObject> RecycleList
    {
      get { return recycleList; }
    }

    public LinkedList<GameObject> NewlyAdded
    {
      get { return newlyAdded; }
    }

    public LinkedList<GameObject> NewlyDeleted
    {
      get { return newlyDeleted; }
    }

    public SpriteBatch SpriteBatch
    {
      get { return spriteBatch; }
      set { spriteBatch = value; }
    }

    public bool IsInitialized
    {
      get { return isInitialized; }
    }

    public bool IsCollisionDetection
    {
      get { return isCollisionDetection; }
      set { isCollisionDetection = value; }
    }

    public SpriteBlendMode BlendMode
    {
      get { return blendMode; }
      set { blendMode = value; }
    }

    public SpriteSortMode SortMode
    {
      get { return sortMode; }
      set { sortMode = value; }
    }

    public SaveStateMode StateMode
    {
      get { return stateMode; }
      set { stateMode = value; }
    }

    public Matrix TransformMatrix
    {
      get { return transformMatrix; }
      set { transformMatrix = value; }
    }

    public Camera ActiveCamera
    {
      get { return activeCamera; }
      set { activeCamera = value; }
    }

    public int DrawCount
    {
      get { return drawCount; }
    }

    #endregion
  }
}