using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.GameObjectManagement.BoundingVolumes;
using XQUEST.Input;

namespace XQUEST.GameObjectManagement
{
  public class ModelGameObject : GameObject3D
  {
    #region Fields

    private readonly string modelName;
    private Model model;
    private float scale;
    private Matrix world;
    private Matrix[] transforms;
    private XQBoundingSphere boundingSphere;

    public XQBoundingSphere XQBoundingSphere
    {
      get { return boundingSphere; }
    }

    #endregion

    #region Constructors

    public ModelGameObject(Game game, string modelName) : base(game)
    {
      this.modelName = modelName;
      scale = 1.0f;

      world = Matrix.Identity;
    }

    #endregion

    #region Virtual Methods

#if !XBOX
    /// <summary>
    /// Should override this method if something's going to happen when the model
    /// gets right clicked on by the mouse.
    /// </summary>
    public virtual void RightClicked()
    {
    }

    /// <summary>
    /// Should override this method if something's going to happen when the model
    /// gets left clicked on by the mouse.
    /// </summary>
    public virtual void LeftClicked()
    {
    }
#endif

    #endregion

    #region Overriden Interface

    public override IBoundingVolume BoundingVolume
    {
      get
      {
        return boundingSphere;
      }
    }

    public override bool CollidesWith(GameObject gameObject)
    {
      if (!(gameObject is ModelGameObject))
        return false;

      return base.CollidesWith(gameObject);
    }

    public override void LoadContent(ContentManager content)
    {
      model = content.Load<Model>(modelName);
      base.LoadContent(content);
    }
    
    public override void Update(GameTime gameTime, InputManager input)
    {
      Matrix scaleMatrix = Matrix.CreateScale(scale);
      Matrix rotationMatrix = Matrix.CreateRotationX(Rotation.X)*
                              Matrix.CreateRotationY(Rotation.Y)*
                              Matrix.CreateRotationZ(Rotation.Z);
      Matrix translationMatrix = Matrix.CreateTranslation(Position);

      world = scaleMatrix*rotationMatrix*translationMatrix;

      // Create combined bounding sphere that encompasses
      // the entire model.
      BoundingSphere sphere = Model.Meshes[0].BoundingSphere;
      for (int i = 1; i < Model.Meshes.Count; i++)
      {
        BoundingSphere meshBoundingSphere = Model.Meshes[i].BoundingSphere;

        BoundingSphere.CreateMerged(ref sphere,
                                    ref meshBoundingSphere,
                                    out sphere);
      }
      
      boundingSphere = 
        new XQBoundingSphere(sphere.Transform(scaleMatrix * translationMatrix));

      base.Update(gameTime, input);
    }

    public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
    {
      transforms = new Matrix[model.Bones.Count];
      model.CopyAbsoluteBoneTransformsTo(transforms);

      foreach (ModelMesh mesh in model.Meshes)
      {
        foreach (BasicEffect effect in mesh.Effects)
        {
          effect.EnableDefaultLighting();

          effect.View = View;
          effect.Projection = Projection;
          effect.World = world*transforms[mesh.ParentBone.Index];
        }
        mesh.Draw();
      }
      base.Draw(gameTime, spriteBatch);
    }

    #endregion

    #region Properties

    public float Scale
    {
      get { return scale; }
      set { scale = value; }
    }

    public Model Model
    {
      get { return model; }
      set { model = value; }
    }

    public Matrix World
    {
      get { return world; }
      set { world = value; }
    }

    #endregion
  }
}