using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.GameObjectManagement.BoundingVolumes;
using XQUEST.Input;
using XQUEST.SpriteAnimation;

namespace XQUEST.GameObjectManagement
{
  public class SpriteGameObject : GameObject2D
  {
    #region Fields

    protected Sprite sprite;
    protected SpriteSheetInfo spriteSheetInfo;

    private bool isPerPixelCollisionDetection;

    #endregion

    #region Constructors

    /// <summary>
    /// Creates a non-animating SpriteGameObject.
    /// </summary>
    /// <param name="game"></param>
    /// <param name="textureName"></param>
    public SpriteGameObject(Game game, string textureName, Rectangle? sourceRect)
      : base(game)
    {
      if (sourceRect == null)
      {
        sprite = new Sprite(textureName);
      }
      else
      {
        Rectangle rect =
          new Rectangle(sourceRect.Value.X, sourceRect.Value.Y,
                        sourceRect.Value.Width, sourceRect.Value.Height);

        sprite = new Sprite(textureName, rect);
      }
    }

    /// <summary>
    /// Creates an animating SpriteGameObject
    /// </summary>
    /// <param name="game"></param>
    /// <param name="spriteSheetInfo"></param>
    public SpriteGameObject(Game game, SpriteSheetInfo spriteSheetInfo) : base(game)
    {
      this.spriteSheetInfo = spriteSheetInfo;
      sprite = new AnimatedSprite(spriteSheetInfo.TextureName);
    }

    public SpriteGameObject(Game game, SpriteSheetInfo spriteSheetInfo,
                            Animation animation)
      : base(game)
    {
      this.spriteSheetInfo = spriteSheetInfo;
      sprite = new AnimatedSprite(spriteSheetInfo, animation);
    }

    #endregion

    #region Properties

    /// <summary>
    /// The sprite instance representing the graphical
    /// portion of the game object.
    /// </summary>
    public Sprite Sprite
    {
      get { return sprite; }
    }

    public SpriteSheetInfo SpriteSheetInfo
    {
      get { return spriteSheetInfo; }
    }

    public bool IsPerPixelCollisionDetection
    {
      get { return isPerPixelCollisionDetection; }
      set { isPerPixelCollisionDetection = value; }
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Check for collisionn 
    /// </summary>
    /// <param name="otherObject"></param>
    /// <returns></returns>
    private bool PerPixelIntersects(SpriteGameObject otherObject)
    {
      Matrix thisTransform =
        Matrix.CreateTranslation(new Vector3(-this.Sprite.Origin, 0.0f))*
        Matrix.CreateScale(new Vector3(this.Sprite.Scale, 1.0f))*
        Matrix.CreateRotationZ(this.Sprite.Rotation)*
        Matrix.CreateTranslation(new Vector3(this.Position, 0.0f));

      Color[] thisColorData = new Color[this.Sprite.Width*this.Sprite.Height];
      this.Sprite.Texture.GetData<Color>(0, this.Sprite.SourceRect, thisColorData, 0,
                                    thisColorData.Length);

      Matrix otherTransform =
        Matrix.CreateTranslation(new Vector3(-otherObject.Sprite.Origin, 0.0f))*
        Matrix.CreateScale(new Vector3(otherObject.Sprite.Scale, 1.0f))*
        Matrix.CreateRotationZ(otherObject.Sprite.Rotation)*
        Matrix.CreateTranslation(new Vector3(otherObject.Position, 0.0f));

      Color[] otherColorData =
        new Color[otherObject.Sprite.Width*otherObject.Sprite.Height];
      otherObject.Sprite.Texture.GetData<Color>(0, otherObject.Sprite.SourceRect,
                                                otherColorData, 0,
                                                otherColorData.Length);

      return
        IntersectPixels(thisTransform, this.Sprite.SourceRect.Width,
                        this.Sprite.SourceRect.Height,
                        thisColorData,
                        otherTransform, otherObject.Sprite.SourceRect.Width,
                        otherObject.Sprite.SourceRect.Height,
                        otherColorData);
    }

    /// <summary>
    /// Determines if there is overlap of the non-transparent pixels between two
    /// sprites.
    /// </summary>
    /// <param name="transformA">World transform of the first sprite.</param>
    /// <param name="widthA">Width of the first sprite's texture.</param>
    /// <param name="heightA">Height of the first sprite's texture.</param>
    /// <param name="dataA">Pixel color data of the first sprite.</param>
    /// <param name="transformB">World transform of the second sprite.</param>
    /// <param name="widthB">Width of the second sprite's texture.</param>
    /// <param name="heightB">Height of the second sprite's texture.</param>
    /// <param name="dataB">Pixel color data of the second sprite.</param>
    /// <returns>True if non-transparent pixels overlap; false otherwise</returns>
    public static bool IntersectPixels(
      Matrix transformA, int widthA, int heightA, Color[] dataA,
      Matrix transformB, int widthB, int heightB, Color[] dataB)
    {
      // Calculate a matrix which transforms from A's local space into
      // world space and then into B's local space
      Matrix transformAToB = transformA*Matrix.Invert(transformB);

      // When a point moves in A's local space, it moves in B's local space with a
      // fixed direction and distance proportional to the movement in A.
      // This algorithm steps through A one pixel at a time along A's X and Y axes
      // Calculate the analogous steps in B:
      Vector2 stepX = Vector2.TransformNormal(Vector2.UnitX, transformAToB);
      Vector2 stepY = Vector2.TransformNormal(Vector2.UnitY, transformAToB);

      // Calculate the top left corner of A in B's local space
      // This variable will be reused to keep track of the start of each row
      Vector2 yPosInB = Vector2.Transform(Vector2.Zero, transformAToB);

      // For each row of pixels in A
      for (int yA = 0; yA < heightA; yA++)
      {
        // Start at the beginning of the row
        Vector2 posInB = yPosInB;

        // For each pixel in this row
        for (int xA = 0; xA < widthA; xA++)
        {
          // Round to the nearest pixel
          int xB = (int) Math.Round(posInB.X);
          int yB = (int) Math.Round(posInB.Y);

          // If the pixel lies within the bounds of B
          if (0 <= xB && xB < widthB &&
              0 <= yB && yB < heightB)
          {
            // Get the colors of the overlapping pixels
            Color colorA = dataA[xA + yA*widthA];
            Color colorB = dataB[xB + yB*widthB];

            // If both pixels are not completely transparent,
            if (colorA.A != 0 && colorB.A != 0)
            {
              // then an intersection has been found
              return true;
            }
          }

          // Move to the next pixel in the row
          posInB += stepX;
        }

        // Move to the next row
        yPosInB += stepY;
      }

      // No intersection found
      return false;
    }

    #endregion

    #region Delegated Members

    /// <summary>
    /// The scale of the game object.
    /// </summary>
    public Vector2 Scale
    {
      get { return sprite.Scale; }
      set { sprite.Scale = value; }
    }

    /// <summary>
    /// The rotation of the game object.
    /// </summary>
    public float Rotation
    {
      get { return sprite.Rotation; }
      set { sprite.Rotation = value; }
    }

    public void AddAnimation(string key, Animation animation)
    {
      if (sprite is AnimatedSprite)
        ((AnimatedSprite) sprite).AddAnimation(key, animation);
    }

    public void SetAnimation(string animationKey)
    {
      if (sprite is AnimatedSprite)
        ((AnimatedSprite) sprite).SetAnimation(animationKey);
    }

    public Animation CurrentAnimation
    {
      get { return ((AnimatedSprite) sprite).CurrentAnimation; }
    }

    #endregion

    #region Overriden Interface

    public override bool CollidesWith(GameObject gameObject)
    {
      if (!(gameObject is SpriteGameObject))
        return false;

      bool boundingVolumeCollision = base.CollidesWith(gameObject);
      if (boundingVolumeCollision &&
          (isPerPixelCollisionDetection ||
           ((SpriteGameObject) gameObject).IsPerPixelCollisionDetection))
      {
        return PerPixelIntersects((SpriteGameObject) gameObject);
      }
      return boundingVolumeCollision;
    }

    public override IBoundingVolume BoundingVolume
    {
      get
      {
        // Calculate bounding box.
        Vector3 min = new Vector3((int) (Position.X - sprite.Origin.X),
                                  (int) (Position.Y - sprite.Origin.Y), 0.0f);
        Vector3 max = min + new Vector3((int) (sprite.Width*sprite.Scale.X),
                                        (int) (sprite.Height*sprite.Scale.Y), 0.0f);

        return new XQBoundingBox(min, max);
      }
    }

    public override void LoadContent(ContentManager content)
    {
      sprite.LoadTexture(content);

      if (spriteSheetInfo.FrameWidth <= 0)
        spriteSheetInfo.FrameWidth = sprite.Texture.Width;
      if (spriteSheetInfo.FrameHeight <= 0)
        spriteSheetInfo.FrameHeight = sprite.Texture.Height;
    }

    public override void Update(GameTime gameTime, InputManager input)
    {
      base.Update(gameTime, input);

      if (sprite is AnimatedSprite)
        ((AnimatedSprite) sprite).UpdateAnimation(gameTime);
    }

    public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
    {
      sprite.Draw(spriteBatch, Position);
    }

    #endregion
  }
}