using System.Collections.Generic;
using System.Diagnostics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.Input;

namespace XQUEST.GameStateManagement
{
  /// <summary>
  /// The screen manager is a component which manages one or more GameScreen
  /// instances. It maintains a stack of screens, calls their Update and Draw
  /// methods at the appropriate times, and automatically routes input to the
  /// topmost active screen.
  /// </summary>
  public class ScreenManager : DrawableGameComponent
  {
    private readonly List<GameScreen> screens = new List<GameScreen>();
    private readonly List<GameScreen> screensToUpdate = new List<GameScreen>();

    private InputManager input;

    private SpriteBatch spriteBatch;
    private SpriteFont font;
    private Texture2D blankTexture;

    private bool isInitialized;
    private bool traceEnabled;

    private readonly string menuFontContentPath;

    /// <summary>
    /// Constructs a new screen manager component.
    /// </summary>
    public ScreenManager(Game game, string menuFontContentPath)
      : base(game)
    {
      this.menuFontContentPath = menuFontContentPath;
    }

        /// <summary>
    /// A default SpriteBatch shared by all the screens. This saves
    /// each screen having to bother creating their own local instance.
    /// </summary>
    public SpriteBatch SpriteBatch
    {
      get { return spriteBatch; }
    }

    /// <summary>
    /// A default font shared by all the screens. This saves
    /// each screen having to bother loading their own local copy.
    /// </summary>
    public SpriteFont Font
    {
      get { return font; }
    }

    /// <summary>
    /// The Input Manager.
    /// </summary>
    public InputManager Input
    {
      get { return input; }
    }

    public Texture2D BlankTexture
    {
      get { return blankTexture; }
    }

    /// <summary>
    /// If true, the manager prints out a list of all the screens
    /// each time it is updated. This can be useful for making sure
    /// everything is being added and removed at the right times.
    /// </summary>
    public bool TraceEnabled
    {
      get { return traceEnabled; }
      set { traceEnabled = value; }
    }

    /// <summary>
    /// Initializes the screen manager component.
    /// </summary>
    public override void Initialize()
    {
      base.Initialize();

      input = (InputManager)Game.Services.GetService(typeof(InputManager));
      isInitialized = true;
    }

    /// <summary>
    /// Load your graphics content.
    /// </summary>
    protected override void LoadContent()
    {
      // Load content belonging to the screen manager.
      ContentManager content = Game.Content;

      spriteBatch = new SpriteBatch(GraphicsDevice);
      font = content.Load<SpriteFont>(menuFontContentPath);
      blankTexture = new Texture2D(GraphicsDevice, 1, 1, 1, TextureUsage.None, SurfaceFormat.Color);
      blankTexture.SetData(new Color[] { Color.White });

      // Tell each of the screens to load their content.
      foreach (GameScreen screen in screens)
      {
        screen.LoadContent();
      }
    }

    /// <summary>
    /// Unload your graphics content.
    /// </summary>
    protected override void UnloadContent()
    {
      // Tell each of the screens to unload their content.
      foreach (GameScreen screen in screens)
      {
        screen.UnloadContent();
      }
    }

    /// <summary>
    /// Allows each screen to run logic.
    /// </summary>
    public override void Update(GameTime gameTime)
    {
      // Make a copy of the master screen list, to avoid confusion if
      // the process of updating one screen adds or removes others.
      screensToUpdate.Clear();

      foreach (GameScreen screen in screens)
        screensToUpdate.Add(screen);

      bool otherScreenHasFocus = !Game.IsActive;
      bool coveredByOtherScreen = false;

      // Loop as long as there are screens waiting to be updated.
      while (screensToUpdate.Count > 0)
      {
        // Pop the topmost screen off the waiting list.
        GameScreen screen = screensToUpdate[screensToUpdate.Count - 1];

        screensToUpdate.RemoveAt(screensToUpdate.Count - 1);

        // Update the screen.
        screen.Update(gameTime, otherScreenHasFocus, coveredByOtherScreen);

        if (screen.ScreenState == ScreenState.TransitionOn ||
            screen.ScreenState == ScreenState.Active)
        {
          // If this is the first active screen we came across,
          // give it a chance to handle input.
          if (!otherScreenHasFocus)
          {
            screen.HandleInput(input);

            otherScreenHasFocus = true;
          }

          // If this is an active non-popup, inform any subsequent
          // screens that they are covered by it.
          if (!screen.IsPopup)
            coveredByOtherScreen = true;
        }
      }

      // Print debug trace?
      if (traceEnabled)
        TraceScreens();
    }


    /// <summary>
    /// Prints a list of all the screens, for debugging.
    /// </summary>
    void TraceScreens()
    {
      List<string> screenNames = new List<string>();

      foreach (GameScreen screen in screens)
        screenNames.Add(screen.GetType().Name);

      Trace.WriteLine(string.Join(", ", screenNames.ToArray()));
    }


    /// <summary>
    /// Tells each screen to draw itself.
    /// </summary>
    public override void Draw(GameTime gameTime)
    {
      foreach (GameScreen screen in screens)
      {
        if (screen.ScreenState == ScreenState.Hidden)
          continue;

        screen.Draw(gameTime);
      }
    }

    /// <summary>
    /// Adds a new screen to the screen manager.
    /// </summary>
    public void AddScreen(GameScreen screen)
    {
      screen.ScreenManager = this;
      screen.IsExiting = false;

      // If we have a graphics device, tell the screen to load content.
      if (isInitialized)
      {
        screen.Initialize();
        screen.LoadContent();
      }

      screens.Add(screen);
    }


    /// <summary>
    /// Removes a screen from the screen manager. You should normally
    /// use GameScreen.ExitScreen instead of calling this directly, so
    /// the screen can gradually transition off rather than just being
    /// instantly removed.
    /// </summary> 
    public void RemoveScreen(GameScreen screen)
    {
      // If we have a graphics device, tell the screen to unload content.
      if (isInitialized)
      {
        screen.UnloadContent();
      }

      screens.Remove(screen);
      screensToUpdate.Remove(screen);

    }


    /// <summary>
    /// Expose an array holding all the screens. We return a copy rather
    /// than the real master list, because screens should only ever be added
    /// or removed using the AddScreen and RemoveScreen methods.
    /// </summary>
    public GameScreen[] GetScreens()
    {
      return screens.ToArray();
    }


    /// <summary>
    /// Helper draws a translucent black fullscreen sprite, used for fading
    /// screens in and out, and for darkening the background behind popups.
    /// </summary>
    public void FadeBackBufferToBlack(int alpha)
    {
      Viewport viewport = GraphicsDevice.Viewport;

      spriteBatch.Begin();

      spriteBatch.Draw(blankTexture,
                       new Rectangle(0, 0, viewport.Width, viewport.Height),
                       new Color(0, 0, 0, (byte)alpha));

      spriteBatch.End();
    }
  }
}
