using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using XQUEST.GameStateManagement;
using XQUEST.Input;

namespace XQUEST.GameStateManagement.Screens
{
  /// <summary>
  /// Base class for screens that contain a menu of options. The user can
  /// move up and down to select an entry, or cancel to back out of the screen.
  /// </summary>
  public abstract class MenuScreen : GameScreen
  {
    private List<MenuEntry> menuEntries = new List<MenuEntry>();
    private int selectedEntry = 0;
    private string menuTitle;

    /// <summary>
    /// Gets the list of menu entries, so derived classes can add
    /// or change the menu contents.
    /// </summary>
    protected IList<MenuEntry> MenuEntries
    {
      get { return menuEntries; }
    }

    /// <summary>
    /// Constructor.
    /// </summary>
    public MenuScreen(string menuTitle)
    {
      this.menuTitle = menuTitle;

      TransitionOnTime = TimeSpan.FromSeconds(0.5);
      TransitionOffTime = TimeSpan.FromSeconds(0.5);
    }


    public static bool MenuUp(InputManager input)
    {
      return input.Keyboard.IsKeyPressed(Keys.Up) || input.GamePad.IsButtonPressed(Buttons.DPadUp) ||
             input.GamePad.IsButtonPressed(Buttons.LeftThumbstickUp);
    }

    public static bool MenuDown(InputManager input)
    {
      return input.Keyboard.IsKeyPressed(Keys.Down) || input.GamePad.IsButtonPressed(Buttons.DPadDown) ||
             input.GamePad.IsButtonPressed(Buttons.LeftThumbstickDown);
    }

    public static bool MenuSelect(InputManager input)
    {
      return input.Keyboard.IsKeyPressed(Keys.Enter) || input.GamePad.IsButtonPressed(Buttons.Start) ||
             input.GamePad.IsButtonPressed(Buttons.A);
    }

    public static bool MenuCancel(InputManager input)
    {
      return input.Keyboard.IsKeyPressed(Keys.Escape) || input.GamePad.IsButtonPressed(Buttons.Back) ||
             input.GamePad.IsButtonPressed(Buttons.B);
    }

    /// <summary>
    /// Responds to user input, changing the selected entry and accepting
    /// or cancelling the menu.
    /// </summary>
    public override void HandleInput(InputManager input)
    {
      // Move to the previous menu entry?
      if (MenuUp(input))
      {
        selectedEntry--;

        if (selectedEntry < 0)
          selectedEntry = menuEntries.Count - 1;
      }

      // Move to the next menu entry?
      if (MenuDown(input))
      {
        selectedEntry++;

        if (selectedEntry >= menuEntries.Count)
          selectedEntry = 0;
      }

      // Accept or cancel the menu?
      if (MenuSelect(input))
      {
        OnSelectEntry(selectedEntry);
      }
      else if (MenuCancel(input))
      {
        OnCancel();
      }
    }

    /// <summary>
    /// Handler for when the user has chosen a menu entry.
    /// </summary>
    protected virtual void OnSelectEntry(int entryIndex)
    {
      menuEntries[selectedEntry].OnSelectEntry();
    }

    /// <summary>
    /// Handler for when the user has cancelled the menu.
    /// </summary>
    protected virtual void OnCancel()
    {
      ExitScreen();
    }

    /// <summary>
    /// Helper overload makes it easy to use OnCancel as a MenuEntry event handler.
    /// </summary>
    protected void OnCancel(object sender, EventArgs e)
    {
      OnCancel();
    }

    /// <summary>
    /// Updates the menu.
    /// </summary>
    public override void Update(GameTime gameTime, bool otherScreenHasFocus,
                                bool coveredByOtherScreen)
    {
      base.Update(gameTime, otherScreenHasFocus, coveredByOtherScreen);

      // Update each nested MenuEntry object.
      for (int i = 0; i < menuEntries.Count; i++)
      {
        bool isSelected = IsActive && (i == selectedEntry);

        menuEntries[i].Update(this, isSelected, gameTime);
      }
    }

    /// <summary>
    /// Draws the menu.
    /// </summary>
    public override void Draw(GameTime gameTime)
    {
      SpriteBatch spriteBatch = ScreenManager.SpriteBatch;
      SpriteFont font = ScreenManager.Font;

      Vector2 position = new Vector2(100, 150);

      // Make the menu slide into place during transitions, using a
      // power curve to make things look more interesting (this makes
      // the movement slow down as it nears the end).
      float transitionOffset = (float) Math.Pow(TransitionPosition, 2);

      if (ScreenState == ScreenState.TransitionOn)
        position.X -= transitionOffset*256;
      else
        position.X += transitionOffset*512;

      spriteBatch.Begin();

      // Draw each menu entry in turn.
      for (int i = 0; i < menuEntries.Count; i++)
      {
        MenuEntry menuEntry = menuEntries[i];

        bool isSelected = IsActive && (i == selectedEntry);

        menuEntry.Draw(this, position, isSelected, gameTime);

        position.Y += menuEntry.GetHeight(this);
      }

      // Draw the menu title.
      Vector2 titlePosition = new Vector2(426, 80);
      Vector2 titleOrigin = font.MeasureString(menuTitle)/2;
      Color titleColor = new Color(192, 192, 192, TransitionAlpha);
      float titleScale = 1.25f;

      titlePosition.Y -= transitionOffset*100;

      spriteBatch.DrawString(font, menuTitle, titlePosition, titleColor, 0,
                             titleOrigin, titleScale, SpriteEffects.None, 0);

      spriteBatch.End();
    }
  }
}