using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.Input;

namespace XQUEST.GameStateManagement.Screens
{
  /// <summary>
  /// A popup message box screen, used to display "are you sure?"
  /// confirmation messages.
  /// </summary>
  public class MessageBoxScreen : GameScreen
  {
    private readonly string acceptIconTextureContentPath;
    private readonly string cancelIconTextureContentPath;
    private readonly float iconScale;
    private readonly string acceptText;
    private readonly string cancelText;

    // For displaying icons for the choices one can make in the message box.
    private Texture2D acceptIconTexture;
    private Texture2D cancelIconTexture;

    private string message;

    public event EventHandler<EventArgs> Accepted;
    public event EventHandler<EventArgs> Cancelled;

    public MessageBoxScreen(string message, string acceptIconTextureContentPath,
                            string cancelIconTextureContentPath,
                            float iconScale, string acceptText, string cancelText) : this(message, false)
    {
      this.acceptIconTextureContentPath = acceptIconTextureContentPath;
      this.cancelIconTextureContentPath = cancelIconTextureContentPath;
      this.iconScale = iconScale;
      this.acceptText = acceptText;
      this.cancelText = cancelText;
    }

    /// <summary>
    /// Constructor automatically includes the standard "A=ok, B=cancel"
    /// usage text prompt.
    /// </summary>
    public MessageBoxScreen(string message)
      : this(message, true)
    {
    }

    /// <summary>
    /// Constructor lets the caller specify whether to include the standard
    /// "A=ok, B=cancel" usage text prompt.
    /// </summary>
    public MessageBoxScreen(string message, bool includeUsageText)
    {
      const string usageText = "\nA button, Enter = ok" +
                               "\nB button, Esc = cancel";

      if (includeUsageText)
        this.message = message + usageText;
      else
        this.message = message;

      IsPopup = true;

      TransitionOnTime = TimeSpan.FromSeconds(0.2);
      TransitionOffTime = TimeSpan.FromSeconds(0.2);
    }

    /// <summary>
    /// Loads graphics content for this screen. This uses the shared ContentManager
    /// provided by the Game class, so the content will remain loaded forever.
    /// Whenever a subsequent MessageBoxScreen tries to load this same content,
    /// it will just get back another reference to the already loaded data.
    /// </summary>
    public override void LoadContent()
    {
      ContentManager content = ScreenManager.Game.Content;

      if (acceptIconTextureContentPath != null)
        acceptIconTexture = content.Load<Texture2D>(acceptIconTextureContentPath);

      if (cancelIconTextureContentPath != null)
        cancelIconTexture = content.Load<Texture2D>(cancelIconTextureContentPath);
    }

    /// <summary>
    /// Responds to user input, accepting or cancelling the message box.
    /// </summary>
    public override void HandleInput(InputManager input)
    {
      if (MenuScreen.MenuSelect(input))
      {
        // Raise the accepted event, then exit the message box.
        if (Accepted != null)
          Accepted(this, EventArgs.Empty);

        ExitScreen();
      }
      else if (MenuScreen.MenuCancel(input))
      {
        // Raise the cancelled event, then exit the message box.
        if (Cancelled != null)
          Cancelled(this, EventArgs.Empty);

        ExitScreen();
      }
    }

    /// <summary>
    /// Draws the message box.
    /// </summary>
    public override void Draw(GameTime gameTime)
    {
      SpriteBatch spriteBatch = ScreenManager.SpriteBatch;
      SpriteFont font = ScreenManager.Font;

      // Darken down any other screens that were drawn beneath the popup.
      ScreenManager.FadeBackBufferToBlack(TransitionAlpha*2/3);

      // Center the message text in the viewport.
      Viewport viewport = ScreenManager.GraphicsDevice.Viewport;
      Vector2 viewportSize = new Vector2(viewport.Width, viewport.Height);
      Vector2 textSize = font.MeasureString(message);
      Vector2 textPosition = (viewportSize - textSize)/2;

      // The background includes a border somewhat larger than the text itself.
      const int hPad = 32;
      const int vPad = 16;

      Rectangle backgroundRectangle = new Rectangle((int) textPosition.X - hPad,
                                                    (int) textPosition.Y - vPad,
                                                    (int) textSize.X + hPad*2,
                                                    (int) textSize.Y + vPad*2);

      if (acceptIconTexture != null)
        backgroundRectangle.Height *= 2;

      // Fade the popup alpha during transitions.
      Color color = new Color(255, 255, 255, TransitionAlpha);

      spriteBatch.Begin();

      // Draw the background rectangle.
      spriteBatch.Draw(ScreenManager.BlankTexture, backgroundRectangle, new Color(0, 0, 0, 128));

      // Draw the message box text.
      spriteBatch.DrawString(font, message, textPosition, color);

      if (acceptIconTexture != null)
      {
        Vector2 iconSize = new Vector2(acceptIconTexture.Width * iconScale, acceptIconTexture.Height*iconScale);

        // Draw accept button + text
        Vector2 acceptIconPosition = textPosition;
        acceptIconPosition.Y += textSize.Y + vPad;
        
        Vector2 acceptIconTextPosition = acceptIconPosition;
        acceptIconTextPosition.X += acceptIconTexture.Width*iconScale;

        spriteBatch.Draw(acceptIconTexture, acceptIconPosition, null, Color.White,
          0.0f, Vector2.Zero, iconScale, SpriteEffects.None, 0.0f);
        spriteBatch.DrawString(font, acceptText, acceptIconTextPosition, Color.White);

        // Draw cancel button + text
        Vector2 cancelIconPosition = acceptIconPosition;
        cancelIconPosition.Y += iconSize.Y;

        Vector2 cancelIconTextPosition = cancelIconPosition;
        cancelIconTextPosition.X += cancelIconTexture.Width * iconScale;

        spriteBatch.Draw(cancelIconTexture, cancelIconPosition, null, Color.White,
          0.0f, Vector2.Zero, iconScale, SpriteEffects.None, 0.0f);
        spriteBatch.DrawString(font, cancelText, cancelIconTextPosition, Color.White);
      }

      spriteBatch.End();
    }
  }
}