using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace XQUEST.Helpers
{
  /// <summary>
  /// Helper class for drawing 2D text using sprite fonts.
  /// </summary>
  public static class TextHelper
  {
    // The font to use.
    public static SpriteFont Font;

    // The sprite batch to use.
    public static SpriteBatch SpriteBatch;
    
    /// <summary>
    /// Initializes the class with a SpriteFont object.
    /// </summary>
    /// <param name="spriteFont">SpriteFont object for drawing text with.</param>
    /// <param name="spriteBatch">SpriteBatch object to use for batching up text.</param>
    public static void Initialize(SpriteFont spriteFont, SpriteBatch spriteBatch)
    {
      Font = spriteFont;
      SpriteBatch = spriteBatch;
    }

    /// <summary>
    /// Draws simple text, specifying the sprite batch, text, Position, and text color.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="textColor">The color of the text.</param>
    public static void DrawText(string text, Vector2 position, Color textColor)
    {
      DrawText(text, position, textColor, 0.0f, Vector2.Zero,Vector2.One, SpriteEffects.None, 0.0f);
    }

    /// <summary>
    /// Draws simple text, specifying the sprite batch, text, Position, text color, 
    /// and scale.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="textColor">The color of the text.</param>
    /// <param name="scale">The scale of the text</param>
    public static void DrawText(string text,
                                Vector2 position,
                                Color textColor, Vector2 scale)
    {
      DrawText(text, position, textColor, 0.0f, Vector2.Zero, scale, SpriteEffects.None, 0.0f);
    }

    /// <summary>
    /// Draws simple text, specifying the sprite batch, text, Position, text color,
    /// rotation, origin, scale, sprite effects, and layer depth.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="textColor">The color of the text.</param>
    /// <param name="rotation">The angle of the text, in radians.</param>
    /// <param name="origin">The origin of the string. Specify (0, 0) for the upper-left 
    /// corner.</param>
    /// <param name="scale">The scale of the text.</param>
    /// <param name="effects">The sprite effects of the text.</param>
    /// <param name="layerDepth">The layer depth of the text.</param>
    public static void DrawText(string text,
                                Vector2 position,
                                Color textColor, float rotation, Vector2 origin,
                                Vector2 scale,
                                SpriteEffects effects, float layerDepth)
    {
      if (Font != null && SpriteBatch != null)
      {
        SpriteBatch.DrawString(Font, text, position, textColor, rotation, origin,
                               scale, effects, layerDepth);
      }
    }

    /// <summary>
    /// Draws simple text, centered, specifying the sprite batch, text, Position, 
    /// and text color.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="textColor">The color of the text.</param>
    public static void DrawTextCentered(string text,
                                        Vector2 position, Color textColor)
    {
      DrawTextCentered(text, position, textColor, 0.0f, Vector2.One,
                       SpriteEffects.None, 0.0f);
    }

    /// <summary>
    /// Draws simple text, centered, specifying the sprite batch, text, Position,
    /// text color, and scale.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="textColor">The color of the text.</param>
    /// <param name="scale">The scale of the text.</param>
    public static void DrawTextCentered(string text,
                                        Vector2 position, Color textColor,
                                        Vector2 scale)
    {
      DrawTextCentered(text, position, textColor, 0.0f, scale,
                       SpriteEffects.None, 0.0f);
    }

    /// <summary>
    /// Draws simple text, centered, specifying the sprite batch, text, Position, 
    /// text color, rotation, origin, scale, sprite effects, and layer depth.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="textColor">The color of the text.</param>
    /// <param name="rotation">The angle of the text, in radians.</param>
    /// <param name="scale">The scale of the text.</param>
    /// <param name="effects">The sprite effects of the text.</param>
    /// <param name="layerDepth">The layer depth of the text.</param>
    public static void DrawTextCentered(string text,
                                        Vector2 position, Color textColor,
                                        float rotation,
                                        Vector2 scale,
                                        SpriteEffects effects, float layerDepth)
    {
      if (Font != null && SpriteBatch != null)
      {
        Vector2 textSize = Font.MeasureString(text);
        Vector2 origin = textSize/2;
        SpriteBatch.DrawString(Font, text, position, textColor, rotation,
                               origin, scale, effects, layerDepth);
      }
    }

    /// <summary>
    /// Draws shadowed text, specifying the sprite batch, text, Position, 
    /// shadow color, and text color.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="shadowColor">The color of the shadow.</param>
    /// <param name="textColor">The color of the text.</param>
    public static void DrawShadowedText(string text,
                                        Vector2 position, Color shadowColor,
                                        Color textColor)
    {
      DrawShadowedText(text, position, shadowColor, textColor, 0.0f,
                       Vector2.Zero, Vector2.One, SpriteEffects.None, 0.0f);
    }

    /// <summary>
    /// Draws shadowed text, specifying the sprite batch, text, Position, 
    /// shadow color, text color, and scale.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="shadowColor">The color of the shadow.</param>
    /// <param name="textColor">The color of the text.</param>
    /// <param name="scale">The scale of the text.</param>
    public static void DrawShadowedText(string text,
                                        Vector2 position, Color shadowColor,
                                        Color textColor,
                                        Vector2 scale)
    {
      DrawShadowedText(text, position, shadowColor, textColor, 0.0f,
                       Vector2.Zero, scale, SpriteEffects.None, 0.0f);
    }

    /// <summary>
    /// Draws shadowed text, specifying the sprite batch, text, Position, 
    /// shadow color, text color, rotation, origin, scale, sprite effects, 
    /// and layer depth.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="shadowColor">The color of the shadow</param>
    /// <param name="textColor">The color of the text.</param>
    /// <param name="rotation">The angle of the text, in radians.</param>
    /// <param name="origin">The origin of the string. Specify (0, 0) for the
    /// upper-left corner.</param>
    /// <param name="scale">The scale of the text.</param>
    /// <param name="effects">The sprite effects of the text.</param>
    /// <param name="layerDepth">The layer depth of the text.</param>
    public static void DrawShadowedText(string text,
                                        Vector2 position, Color shadowColor,
                                        Color textColor,
                                        float rotation,
                                        Vector2 origin, Vector2 scale,
                                        SpriteEffects effects,
                                        float layerDepth)
    {
      if (Font != null && SpriteBatch != null)
      {
        SpriteBatch.DrawString(Font, text,
                               new Vector2(position.X - 1, position.Y - 1),
                               shadowColor, rotation, origin, scale, effects,
                               layerDepth);

        SpriteBatch.DrawString(Font, text, position, textColor, rotation,
                               origin, scale, effects, layerDepth);
      }
    }

    /// <summary>
    /// Draws shadowed text, centered, specifying the sprite batch, text, Position, 
    /// shadow color, and text color.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="shadowColor">The color of the shadow.</param>
    /// <param name="textColor">The color of the text.</param>
    public static void DrawShadowedTextCentered(string text,
                                                Vector2 position, Color shadowColor,
                                                Color textColor)
    {
      DrawShadowedTextCentered(text, position, shadowColor, textColor,
                               0.0f, Vector2.One, SpriteEffects.None, 0.0f);
    }

    /// <summary>
    /// Draws shadowed text, centered, specifying the sprite batch, text, Position, 
    /// shadow color, text color, and scale.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="shadowColor">The color of the shadow.</param>
    /// <param name="textColor">The color of the text.</param>
    /// <param name="scale">The scale of the text.</param>
    public static void DrawShadowedTextCentered(string text,
                                                Vector2 position, Color shadowColor,
                                                Color textColor,
                                                Vector2 scale)
    {
      DrawShadowedTextCentered(text, position, shadowColor, textColor,
                               0.0f, scale, SpriteEffects.None, 0.0f);
    }

    /// <summary>
    /// Draws shadowed text, centered, specifying the sprite batch, text, Position, 
    /// shadow color, text color, rotation, origin, scale, sprite effects, 
    /// and layer depth.
    /// </summary>
    /// <param name="text">The text to draw.</param>
    /// <param name="position">The Position of the text in screen coordinates.</param>
    /// <param name="shadowColor">The color of the shadow.</param>
    /// <param name="textColor">The color of the text.</param>
    /// <param name="rotation">The angle of the text, in radians.</param>
    /// <param name="scale">The scale of the text.</param>
    /// <param name="effects">The sprite effects of the text.</param>
    /// <param name="layerDepth">The layer depth of the text.</param>
    public static void DrawShadowedTextCentered(string text,
                                                Vector2 position, Color shadowColor,
                                                Color textColor,
                                                float rotation, Vector2 scale,
                                                SpriteEffects effects,
                                                float layerDepth)
    {
      if (Font != null && SpriteBatch != null)
      {
        Vector2 textSize = Font.MeasureString(text);
        Vector2 origin = textSize/2;

        SpriteBatch.DrawString(Font, text,
                               new Vector2(position.X - 1, position.Y - 1),
                               shadowColor, rotation, origin, scale, effects,
                               layerDepth);

        SpriteBatch.DrawString(Font, text, position, textColor, rotation,
                               origin, scale, effects, layerDepth);
      }
    }
  }
}