using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace XQUEST.Input
{
  public class GamePadInput
  {
    #region Fields

    // Array to hold the current state of each of the four game pads.
    private readonly GamePadState[] currentGamePadStates;

    // Array to hold the state of each of the four game pads from the previous 
    // update call.
    private readonly GamePadState[] lastGamePadStates;

    #endregion

    #region Constructors

    public GamePadInput(int maxInputs)
    {
      currentGamePadStates = new GamePadState[maxInputs];
      lastGamePadStates = new GamePadState[maxInputs];
    }

    #endregion

    #region Public Methods

    public void Update()
    {
      // Save game pad state from the previous update call.
      for (int i = 0; i < currentGamePadStates.Length; i++)
      {
        lastGamePadStates[i] = currentGamePadStates[i];
        currentGamePadStates[i] = GamePad.GetState((PlayerIndex)i);
      }
    }

    /// <summary>
    /// Returns whether the specified game pad is connected.
    /// </summary>
    /// <param name="playerIndex">Enumerator value that specifies which game 
    /// pad to query.</param>
    /// <returns>true if the game pad is connected, false otherwise.</returns>
    public bool GamePadIsConnected(PlayerIndex playerIndex)
    {
      return currentGamePadStates[(int)playerIndex].IsConnected;
    }

    public bool IsAnyButtonDown()
    {
      for (int i = 0; i < currentGamePadStates.Length; i++)
      {
        if (IsAnyButtonDown((PlayerIndex)i))
          return true;
      }
      return false;
    }

    public bool IsAnyButtonDown(PlayerIndex playerIndex)
    {
      int i = (int)playerIndex;
      
      return currentGamePadStates[i].IsButtonDown(Buttons.A) ||
             currentGamePadStates[i].IsButtonDown(Buttons.B) ||
             currentGamePadStates[i].IsButtonDown(Buttons.X) ||
             currentGamePadStates[i].IsButtonDown(Buttons.Y) ||
             currentGamePadStates[i].IsButtonDown(Buttons.Start) ||
             currentGamePadStates[i].IsButtonDown(Buttons.Back) ||
             currentGamePadStates[i].IsButtonDown(Buttons.DPadDown) ||
             currentGamePadStates[i].IsButtonDown(Buttons.DPadLeft) ||
             currentGamePadStates[i].IsButtonDown(Buttons.DPadRight) ||
             currentGamePadStates[i].IsButtonDown(Buttons.DPadUp) ||
             currentGamePadStates[i].IsButtonDown(Buttons.LeftShoulder) ||
             currentGamePadStates[i].IsButtonDown(Buttons.RightShoulder) ||
             currentGamePadStates[i].IsButtonDown(Buttons.LeftStick) ||
             currentGamePadStates[i].IsButtonDown(Buttons.RightStick) ||
             currentGamePadStates[i].IsButtonDown(Buttons.LeftTrigger) ||
             currentGamePadStates[i].IsButtonDown(Buttons.RightTrigger);
    }

    public bool IsButtonDown(Buttons button)
    {
      for (int i = 0; i < currentGamePadStates.Length; i++)
      {
        if (IsButtonDown(button, (PlayerIndex)i))
          return true;
      }
      return false;
    }

    public bool IsButtonDown(Buttons button, PlayerIndex playerIndex)
    {
      return currentGamePadStates[(int)playerIndex].IsButtonDown(button);
    }

    public bool IsButtonPressed(Buttons button)
    {
      for (int i = 0; i < currentGamePadStates.Length; i++)
      {
        if (IsButtonPressed(button, (PlayerIndex)i))
          return true;
      }
      return false;
    }

    public bool IsButtonPressed(Buttons button, PlayerIndex playerIndex)
    {
      return currentGamePadStates[(int)playerIndex].IsButtonDown(button)
             && lastGamePadStates[(int)playerIndex].IsButtonUp(button);
    }

    // These methods gets Position values for the thumb sticks and triggers

    /// <summary>
    /// Gets the Position of the game pad left thumbstick.
    /// </summary>
    /// <param name="playerIndex">Which of the 4 gamepads to query.</param>
    /// <returns>The Position of the game pad left thumbstick.</returns>
    public Vector2 GetLeftStickPosition(PlayerIndex playerIndex)
    {
      return currentGamePadStates[(int)playerIndex].ThumbSticks.Left;
    }

    /// <summary>
    /// Gets the Position of the game pad left thumbstick (Player One).
    /// </summary>
    /// <returns>The Position of the game pad left thumbstick (Player One).</returns>
    public Vector2 GetLeftStickPosition()
    {
      return currentGamePadStates[(int)PlayerIndex.One].ThumbSticks.Left;
    }

    /// <summary>
    /// Gets the Position of the game pad right thumbstick.
    /// </summary>
    /// <param name="playerIndex">Which of the 4 gamepads to query.</param>
    /// <returns>The Position of the game pad right thumbstick</returns>
    public Vector2 GetRightStickPosition(PlayerIndex playerIndex)
    {
      return currentGamePadStates[(int)playerIndex].ThumbSticks.Right;
    }

    /// <summary>
    /// Gets the Position of the game pad right thumbstick (Player One).
    /// </summary>
    /// <returns>The Position of the game pad right thumbstick (Player One).</returns>
    public Vector2 GetRightStickPosition()
    {
      return currentGamePadStates[(int)PlayerIndex.One].ThumbSticks.Right;
    }

    /// <summary>
    /// Gets the Position of the game pad left trigger.
    /// </summary>
    /// <param name="playerIndex">Which of the 4 gamepads to query.</param>
    /// <returns>The Position of the game pad left trigger.</returns>
    public float GetLeftTriggerPosition(PlayerIndex playerIndex)
    {
      return currentGamePadStates[(int)playerIndex].Triggers.Left;
    }

    /// <summary>
    /// Gets the Position of the game pad left trigger (Player One).
    /// </summary>
    /// <returns>The Position of the game pad left trigger (Player One).</returns>
    public float GetLeftTriggerPosition()
    {
      return currentGamePadStates[(int)PlayerIndex.One].Triggers.Left;
    }

    /// <summary>
    /// Gets the Position of the game pad right trigger.
    /// </summary>
    /// <param name="playerIndex">Which of the 4 gamepads to query.</param>
    /// <returns>The Position of the game pad right trigger.</returns>
    public float GetRightTriggerPosition(PlayerIndex playerIndex)
    {
      return currentGamePadStates[(int)playerIndex].Triggers.Right;
    }

    /// <summary>
    /// Gets the Position of the game pad right trigger (Player One).
    /// </summary>
    /// <returns>The Position of the game pad right trigger (Player One).</returns>
    public float GetRightTriggerPosition()
    {
      return currentGamePadStates[(int)PlayerIndex.One].Triggers.Right;
    }

    #endregion

    #region Properties

    public GamePadState[] CurrentGamePadStates
    {
      get { return currentGamePadStates; }
    }

    public GamePadState[] LastGamePadStates
    {
      get { return lastGamePadStates; }
    }

    #endregion
  }
}