using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace XQUEST.Input
{
  /// <summary>
  /// Handles all input needs from the keyboard, mouse (Windows only), and
  /// up to four Xbox 360 controllers.
  /// </summary>
  public class InputManager : GameComponent
  {
    #region Fields

    private const int MaxInputs = 4;

    private KeyboardInput keyboard;
    private GamePadInput gamePad;

#if !XBOX
    private MouseInput mouse;
#endif

    #endregion

    #region Constructors

    /// <summary>
    /// Creates a new InputManager. Adds itself as a service.
    /// </summary>
    /// <param name="game">The Game to run under.</param>
    public InputManager(Game game) : base(game)
    {
      keyboard = new KeyboardInput(MaxInputs);
      gamePad = new GamePadInput(MaxInputs);
      mouse = new MouseInput(); 

      // Register the InputManager with the Game Services. Users of the 
      // InputManager should look it up from the Game Services when needed, 
      // and not create it through this constructor.
      game.Services.AddService(typeof (InputManager), this);
      
      // Make sure that out of all components currently in the game,
      // the InputManager is updated first, so that all other components
      // who use InputManager will receive updated input from it.
      UpdateOrder = -1000;
    }

    #endregion

    #region Public Methods

    public override void Update(GameTime gameTime)
    {
      keyboard.Update();
      gamePad.Update();

#if !XBOX
      mouse.Update();
#endif
    }

    /// <summary>
    /// Detects whether the specified player has a key or game pad button down.
    /// </summary>
    /// <param name="key">The key to check.</param>
    /// <param name="button">The button to check.</param>
    /// <param name="playerIndex">Index of the player to check.</param>
    /// <returns></returns>
    public bool IsKeyOrButtonDown(Keys key, Buttons button, PlayerIndex playerIndex)
    {
      return keyboard.IsKeyDown(key, playerIndex) || gamePad.IsButtonDown(button, playerIndex);
    }

    /// <summary>
    /// Detects whether a key or game pad button is down, on any player.
    /// </summary>
    /// <param name="key">The key to check.</param>
    /// <param name="button">The button to check.</param>
    /// <returns></returns>
    public bool IsKeyOrButtonDown(Keys key, Buttons button)
    {
      for (int i = 0; i < MaxInputs; i++)
      {
        if (IsKeyOrButtonDown(key, button, (PlayerIndex) i))
          return true;
      }
      return false;
    }

    /// <summary>
    /// Detects whether the specified player just pressed a key or game pad button.
    /// </summary>
    /// <param name="key">The key to check.</param>
    /// <param name="button">The button to check.</param>
    /// <param name="playerIndex">Index of the player to check.</param>
    /// <returns></returns>
    public bool IsKeyOrButtonPressed(Keys key, Buttons button, PlayerIndex playerIndex)
    {
      return keyboard.IsKeyPressed(key, playerIndex) || gamePad.IsButtonPressed(button, playerIndex);
    }

    /// <summary>
    /// Detects whether a key or game pad button was just pressed, on any player.
    /// </summary>
    /// <param name="key">The key to check.</param>
    /// <param name="button">The button to check.</param>
    /// <returns></returns>
    public bool IsKeyOrButtonPressed(Keys key, Buttons button)
    {
      for (int i = 0; i < MaxInputs; i++)
      {
        if (IsKeyOrButtonPressed(key, button, (PlayerIndex) i))
          return true;
      }
      return false;
    }

    #endregion

    #region Properties

    /// <summary>
    /// Provides access to keyboard input querying.
    /// </summary>
    public KeyboardInput Keyboard
    {
      get { return keyboard; }
    }

    /// <summary>
    /// Provides access to game pad input querying.
    /// </summary>
    public GamePadInput GamePad
    {
      get { return gamePad; }
    }

#if !XBOX
    /// <summary>
    /// Provides access to mouse input querying.
    /// </summary>
    public MouseInput Mouse
    {
      get { return mouse; }
    }
#endif

    #endregion
  }
}