using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace XQUEST.Input
{
  public class KeyboardInput
  {
    #region Fields

    // Keyboard current state and state from the previous update call.
    private readonly KeyboardState[] currentKeyboardStates;
    private readonly KeyboardState[] lastKeyboardStates;

    #endregion

    #region Constructors

    public KeyboardInput(int maxInputs)
    {
      currentKeyboardStates = new KeyboardState[maxInputs];
      lastKeyboardStates = new KeyboardState[maxInputs];
    }

    #endregion

    #region Public Methods

    public void Update()
    {
      for (int i = 0; i < currentKeyboardStates.Length; i++)
      {
        lastKeyboardStates[i] = currentKeyboardStates[i];
        currentKeyboardStates[i] = Keyboard.GetState((PlayerIndex)i);
      }
    }

    /// <summary>
    /// Checks for any key press, by any player.
    /// </summary>
    /// <returns>A boolean value indicating whether any key is down, by any player.</returns>
    public bool IsAnyKeyDown()
    {
      for (int i = 0; i < currentKeyboardStates.Length; i++)
      {
        if (IsAnyKeyDown((PlayerIndex)i))
          return true;
      }
      return false;
    }

    public bool IsAnyKeyDown(PlayerIndex playerIndex)
    {
      return currentKeyboardStates[(int)playerIndex].GetPressedKeys().Length != 0;
    }

    /// <summary>
    /// Checks if a specific key is down, by any player.
    /// </summary>
    /// <param name="key">The key to check.</param>
    /// <returns>A boolean value indicating whether the key was down, by any player.</returns>
    public bool IsKeyDown(Keys key)
    {
      for (int i = 0; i < currentKeyboardStates.Length; i++)
      {
        if (IsKeyDown(key, (PlayerIndex)i))
          return true;
      }
      return false;
    }

    /// <summary>
    /// Checks if a specific key is down.
    /// </summary>
    /// <param name="key">The key to check.</param>
    /// <param name="playerIndex">The player index.</param>
    /// <returns>A boolean value indicating whether the key was down.</returns>
    public bool IsKeyDown(Keys key, PlayerIndex playerIndex)
    {
      return currentKeyboardStates[(int)playerIndex].IsKeyDown(key);
    }

    /// <summary>
    /// Returns whether a key has just been pressed, by any player. This is useful
    /// if you do not want the key to repeat (like in menu selections).
    /// </summary>
    /// <param name="key">Enumerated value that specifies the key to query.</param>
    /// <returns>true if the key was just pressed, false otherwise.</returns>
    public bool IsKeyPressed(Keys key)
    {
      for (int i = 0; i < currentKeyboardStates.Length; i++)
      {
        if (IsKeyPressed(key, (PlayerIndex)i))
          return true;
      }
      return false;
    }

    public bool IsKeyPressed(Keys key, PlayerIndex playerIndex)
    {
      return
        currentKeyboardStates[(int)playerIndex].IsKeyDown(key) &&
        lastKeyboardStates[(int)playerIndex].IsKeyUp(key);
    }

    #endregion

    #region Properties

    public KeyboardState[] CurrentKeyboardStates
    {
      get { return currentKeyboardStates; }
    }

    public KeyboardState[] LastKeyboardStates
    {
      get { return lastKeyboardStates; }
    }

    #endregion
  }
}