using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace XQUEST.Input
{
  public class MouseInput
  {
    #region Fields

    // Mouse current state and state from the previous update call.
    private MouseState currentMouseState = Mouse.GetState();
    private MouseState lastMouseState;

    #endregion

    #region Public Methods

    public void Update()
    {
      // Save last and get the new mouse state.
      lastMouseState = currentMouseState;
      currentMouseState = Mouse.GetState();
    }
    
    #endregion

    #region Properties

    public MouseState CurrentMouseState
    {
      get { return currentMouseState; }
    }

    public MouseState LastMouseState
    {
      get { return lastMouseState; }
    }

    /// <summary>
    /// Gets a boolean value indicating whether the left mouse button is 
    /// being held down.
    /// </summary>
    public bool IsLeftButtonDown
    {
      get { return currentMouseState.LeftButton == ButtonState.Pressed; }
    }

    /// <summary>
    /// Gets a boolean value indicating whether the right mouse button is 
    /// being held down.
    /// </summary>
    public bool IsRightButtonDown
    {
      get { return currentMouseState.RightButton == ButtonState.Pressed; }
    }

    /// <summary>
    /// Gets a boolean value indicating whether the middle mouse button is 
    /// being held down.
    /// </summary>
    public bool IsMiddleButtonDown
    {
      get { return currentMouseState.MiddleButton == ButtonState.Pressed; }
    }

    /// <summary>
    /// Gets a boolean value indicating whether mouse XButton1 is being held down.
    /// </summary>
    public bool IsXButton1Down
    {
      get { return currentMouseState.XButton1 == ButtonState.Pressed; }
    }

    /// <summary>
    /// Gets a boolean value indicating whether mouse XButton2 is being held down.
    /// </summary>
    public bool IsXButton2Down
    {
      get { return currentMouseState.XButton2 == ButtonState.Pressed; }
    }

    /// <summary>
    /// Gets a boolean value indicating whether the left mouse button was just 
    /// pressed.
    /// </summary>
    public bool IsLeftButtonPressed
    {
      get
      {
        return currentMouseState.LeftButton == ButtonState.Pressed &&
               lastMouseState.LeftButton == ButtonState.Released;
      }
    }

    /// <summary>
    /// Gets a boolean value indicating whether the right mouse button was just 
    /// pressed.
    /// </summary>
    public bool IsRightButtonPressed
    {
      get
      {
        return currentMouseState.RightButton == ButtonState.Pressed &&
               lastMouseState.RightButton == ButtonState.Released;
      }
    }

    /// <summary>
    /// Gets a boolean value indicating whether the middle mouse button was just 
    /// pressed.
    /// </summary>
    public bool IsMiddleButtonPressed
    {
      get
      {
        return currentMouseState.MiddleButton == ButtonState.Pressed &&
               lastMouseState.MiddleButton == ButtonState.Released;
      }
    }

    /// <summary>
    /// Gets a boolean value indicating whether mouse XButton1 was just pressed.
    /// </summary>
    public bool IsXButton1Pressed
    {
      get
      {
        return currentMouseState.XButton1 == ButtonState.Pressed &&
               lastMouseState.XButton1 == ButtonState.Released;
      }
    }

    /// <summary>
    /// Gets a boolean value indicating whether mouse XButton2 was just pressed.
    /// </summary>
    public bool IsXButton2Pressed
    {
      get
      {
        return currentMouseState.XButton2 == ButtonState.Pressed &&
               lastMouseState.XButton2 == ButtonState.Released;
      }
    }

    /// <summary>
    /// Gets the Position of the mouse cursor.
    /// </summary>
    public Vector2 Position
    {
      get { return new Vector2(currentMouseState.X, currentMouseState.Y); }
    }

    /// <summary>
    /// Get the amount the mouse moved last frame.
    /// </summary>
    public Vector2 PositionDelta
    {
      get { return Position - new Vector2(lastMouseState.X, lastMouseState.Y); }
    }

    /// <summary>
    /// Gets the horizontal Position of the mouse.
    /// </summary>
    public int PositionX
    {
      get { return currentMouseState.X; }
    }

    /// <summary>
    /// Gets the vertical Position of the mouse.
    /// </summary>
    public int PositionY
    {
      get { return currentMouseState.Y; }
    }

    /// <summary>
    /// Gets the cumulated mouse scroll wheel value since the game started.
    /// </summary>
    public int ScrollWheelValue
    {
      get { return currentMouseState.ScrollWheelValue; }
    }

    /// <summary>
    /// Gets the change in mouse scroll wheel value since last frame.
    /// </summary>
    public int ScrollWheelDelta
    {
      get
      {
        return currentMouseState.ScrollWheelValue -
               lastMouseState.ScrollWheelValue;
      }
    }

    #endregion
  }
}