using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.CameraSystem;

namespace XQUEST.Helpers
{
  /// <summary>
  /// This class draws a square Grid in the XZ-plane centered in the origo by default. 
  /// Needs a camera manager in the game services, and at least one camera added to the manager.
  /// </summary>
  public class Grid3D : DrawableGameComponent
  {
    private BasicEffect basicEffect;
    private VertexDeclaration vertexDeclaration;

    private VertexPositionColor[] verticesZ;
    private VertexPositionColor[] verticesX;

    private int lines;
    private float gridSpacing = 1f;
    private float width = 10f;
    private Vector3 position = Vector3.Zero;

    /// <summary>
    /// Center of the grid in world coordinates. Default is origo.
    /// </summary>
    public Vector3 Position
    {
      get { return position; }
      set { position = value; }
    }

    /// <summary>
    /// Length of one side of the square grid in world units. Default is 10f.
    /// </summary>
    public float Width
    {
      get { return width; }
      set { width = value; }
    }

    /// <summary>
    /// Space between grid lines in world units. Default is 1f.
    /// </summary>
    public float GridSpacing
    {
      get { return gridSpacing; }
      set { gridSpacing = value; }
    }

    public Grid3D(Game game)
      : base(game)
    {
    }

    protected override void LoadContent()
    {
      base.LoadContent();

      basicEffect = new BasicEffect(GraphicsDevice, null);
      basicEffect.VertexColorEnabled = true;

      vertexDeclaration =
        new VertexDeclaration(GraphicsDevice, VertexPositionColor.VertexElements);

      // Create the grid
      lines = (int) (width/gridSpacing) + 1;

      verticesX = new VertexPositionColor[lines*2];
      verticesZ = new VertexPositionColor[lines*2];

      for (int i = 0, k = 0; i < verticesX.Length; i = i + 2, k++)
      {
        verticesX[i].Position =
          new Vector3(k*gridSpacing - (width/2), 0, -(width/2)) + position;
        verticesX[i].Color = Color.Red;
        verticesX[i + 1].Position =
          new Vector3(k*gridSpacing - (width/2), 0, (width/2)) + position;
        verticesX[i + 1].Color = Color.Red;

        verticesZ[i].Position =
          new Vector3(-(width/2), 0, k*gridSpacing - (width/2)) + position;
        verticesZ[i].Color = Color.Red;
        verticesZ[i + 1].Position =
          new Vector3((width/2), 0, k*gridSpacing - (width/2)) + position;
        verticesZ[i + 1].Color = Color.Red;
      }
    }

    public override void Draw(GameTime gameTime)
    {
      base.Draw(gameTime);

      CameraManager cameraManager =
        (CameraManager) Game.Services.GetService(typeof (CameraManager));

      if (cameraManager != null)
      {
        if (cameraManager.ActiveCamera != null)
        {
          // get projection and view matrix from camera
          basicEffect.Projection = cameraManager.ActiveCamera.Projection;
          basicEffect.View = cameraManager.ActiveCamera.View;

          // draw primitives
          GraphicsDevice.VertexDeclaration = vertexDeclaration;

          basicEffect.Begin();
          basicEffect.CurrentTechnique.Passes[0].Begin();

          GraphicsDevice.DrawUserPrimitives(PrimitiveType.LineList, verticesX, 0,
                                            lines);
          GraphicsDevice.DrawUserPrimitives(PrimitiveType.LineList, verticesZ, 0,
                                            lines);

          basicEffect.CurrentTechnique.Passes[0].End();
          basicEffect.End();
        }
        else
        {
          throw new Exception("need to add a camera to the cameramanager");
        }
      }
      else
      {
        throw new Exception("need to add a cameramanager to the game services");
      }
    }
  }
}