using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.CameraSystem;

namespace XQUEST.Misc
{
  /// <summary>
  /// This class draws a ray in the world.
  /// </summary>
  public class RayDrawer : DrawableGameComponent
  {
    #region Fields
    private BasicEffect basicEffect;
    private VertexDeclaration vertexDeclaration;

    private List<VertexPositionColor[]> verticeLists;

    private float rayLength;
    #endregion

    #region Properties
    public float RayLength
    {
      get { return rayLength; }
      set { rayLength = value; }
    }
    #endregion

    #region Constructor
    public RayDrawer(Game game)
      : base(game)
    {
      // default to 1000
      rayLength = 1000f;

      // initiate list
      verticeLists = new List<VertexPositionColor[]>();

      game.Services.AddService(typeof(RayDrawer), this);
    }
    #endregion

    #region Methods
    /// <summary>
    /// Adds a ray to the collection of rays beeing drawn.
    /// </summary>
    /// <param name="ray">A ray defined in world coords</param>
    public void Add(Ray ray)
    {
      verticeLists.Add(CreateVertices(ray));
    }

    /// <summary>
    /// Creates vertices from the ray
    /// </summary>
    /// <param name="ray">the ray to be drawn</param>
    /// <returns></returns>
    private VertexPositionColor[] CreateVertices(Ray ray)
    {
      VertexPositionColor[] vertices = new VertexPositionColor[2];
      
      vertices[0].Position = ray.Position;
      vertices[0].Color = Color.Green;
      vertices[1].Position = ray.Direction * RayLength;
      vertices[1].Color = Color.Green;
      
      return vertices;
    }
    /// <summary>
    /// Clear the collection of rays beeing drawn
    /// </summary>
    public void Clear()
    {
      verticeLists.Clear();
    }

    #endregion

    #region Override methods
    protected override void LoadContent()
    {
      base.LoadContent();

      basicEffect = new BasicEffect(GraphicsDevice, null);
      basicEffect.VertexColorEnabled = true;

      vertexDeclaration =
        new VertexDeclaration(GraphicsDevice, VertexPositionColor.VertexElements);
    }

    public override void Draw(GameTime gameTime)
    {
      base.Draw(gameTime);

      CameraManager cameraManager =
        (CameraManager)Game.Services.GetService(typeof(CameraManager));

      if (cameraManager != null)
      {
        if (cameraManager.ActiveCamera != null)
        {
          if(verticeLists.Count != 0)
          {
            
            // get projection and view matrix from camera
            basicEffect.Projection = cameraManager.ActiveCamera.Projection;
            basicEffect.View = cameraManager.ActiveCamera.View;

            // draw primitives
            GraphicsDevice.VertexDeclaration = vertexDeclaration;

            basicEffect.Begin();
            basicEffect.CurrentTechnique.Passes[0].Begin();

            foreach (VertexPositionColor[] vertices in verticeLists)
            {
              GraphicsDevice.DrawUserPrimitives(PrimitiveType.LineList, vertices, 0, 1);
            }

            basicEffect.CurrentTechnique.Passes[0].End();
            basicEffect.End();
          }
        }
        else
        {
          throw new Exception("need to add a camera to the cameramanager");
        }
      }
      else
      {
        throw new Exception("need to add a cameramanager to the game services");
      }
    }
    #endregion
  }
}