using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using XQUEST.CameraSystem;
using System.Collections.Generic;

namespace XQUEST.Misc
{
  /// <summary>
  /// This game component draws spheres in the world. The game component adds itself
  /// as a game service. Use that instance to add spheres to be drawn.
  /// </summary>
  public class SphereDrawer : DrawableGameComponent
  {
    #region Fields
    private BasicEffect basicEffect;
    private VertexDeclaration vertexDeclaration;

    private List<VertexPositionColor[]> verticeLists;

    private int resolution;
    private Color color;
    #endregion

    #region Properties
    /// <summary>
    /// Set or get the color of the sphere
    /// </summary>
    public Color VertexColor
    {
      get { return color; }
      set { color = value; }
    }
    /// <summary>
    /// Set the resolution for the sphere. Default is 5.
    /// should not set it much above 5 because of major performance impact.
    /// </summary>
    public int Resolution
    {
      get { return resolution; }
      set { resolution = value; }
    }
    #endregion

    #region Constructor
    /// <summary>
    /// Constructor.
    /// </summary>
    /// <param name="game">The game instance</param>
    public SphereDrawer(Game game)
      : base(game)
    {
      // defaults
      resolution = 5;
      color = Color.Black;
      
      // init list
      verticeLists = new List<VertexPositionColor[]>();

      game.Services.AddService(typeof(SphereDrawer), this);
    }
    #endregion

    #region Methods

    /// <summary>
    /// Adds a boundingSphere to the collection of spheres beeing drawn.
    /// </summary>
    /// <param name="sphere">A boundingsphere, with a center in world coords</param>
    public void Add(BoundingSphere sphere)
    {
      verticeLists.Add(CreateVertices(sphere));
    }
    /// <summary>
    /// Clear the collection of spheres beeing drawn
    /// </summary>
    public void Clear()
    {
      verticeLists.Clear();
    }
    /// <summary>
    /// Creates vertices based on sphere's center and the radius.
    /// How many vertices is based on the set resolution.
    /// </summary>
    /// <param name="sphere">The sphere to be drawn</param>
    /// <returns>verteices with position and color</returns>
    private VertexPositionColor[] CreateVertices(BoundingSphere sphere)
    {
      double step = MathHelper.Pi / Math.Pow(2.0, resolution);
      int pointsOnCircle = (int)Math.Pow(2.0, resolution + 1);

      VertexPositionColor[] vertices = new VertexPositionColor[pointsOnCircle * pointsOnCircle];

      float r = sphere.Radius;

      int index = 0;
      for (int i = 0; i < pointsOnCircle; i++)
      {
        for (int k = 0; k < pointsOnCircle; k++)
        {
          float x = sphere.Center.X + (float)(r * Math.Cos(k * step) * Math.Sin(i * step));
          float y = sphere.Center.Y + (float)(r * Math.Sin(k * step) * Math.Sin(i * step));
          float z = sphere.Center.Z + (float)(r * Math.Cos(i * step));
          vertices[index].Position = new Vector3(x, y, z);
          vertices[index].Color = VertexColor;
          index++;
        }
      }

      return vertices;
    }

    #endregion

    #region Override methods
    /// <summary>
    /// initiates basic effect and vertex declaration
    /// </summary>
    protected override void LoadContent()
    {
      base.LoadContent();

      basicEffect = new BasicEffect(GraphicsDevice, null);
      basicEffect.VertexColorEnabled = true;

      vertexDeclaration =
        new VertexDeclaration(GraphicsDevice, VertexPositionColor.VertexElements);
    }

    /// <summary>
    /// Draws the spheres from the vertice lists.
    /// </summary>
    /// <param name="gameTime"></param>
    public override void Draw(GameTime gameTime)
    {
      base.Draw(gameTime);

      CameraManager cameraManager =
        (CameraManager)Game.Services.GetService(typeof(CameraManager));

      if (cameraManager != null)
      {
        if (cameraManager.ActiveCamera != null)
        {
          if (verticeLists.Count != 0)
          {
            // get projection and view matrix from camera
            basicEffect.Projection = cameraManager.ActiveCamera.Projection;
            basicEffect.View = cameraManager.ActiveCamera.View;

            // draw primitives
            GraphicsDevice.VertexDeclaration = vertexDeclaration;

            basicEffect.Begin();
            basicEffect.CurrentTechnique.Passes[0].Begin();

            foreach(VertexPositionColor[] vertices in verticeLists) 
            {
              GraphicsDevice.DrawUserPrimitives(PrimitiveType.PointList, vertices, 0, vertices.Length);
            }
              
            basicEffect.CurrentTechnique.Passes[0].End();
            basicEffect.End();
            
          }
        }
        else
        {
          throw new Exception("need to add a camera to the cameramanager");
        }
      }
      else
      {
        throw new Exception("need to add a cameramanager to the game services");
      }
    }
    #endregion
  }
}