using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace XQUEST.SpriteAnimation
{
  /// <summary>
  /// Represents a sequence of images rendered after eachother
  /// to simulate animation.
  /// </summary>
  public class AnimatedSprite : Sprite
  {
    #region Fields

    // Dictionary of Animation objects, keyed by a string describing the object.
    private Dictionary<string, Animation> animations =
      new Dictionary<string, Animation>();

    private float elapsedTime;
    private string currentAnimationKey;

    #endregion

    #region Constructors

    /// <summary>
    /// Creates a new AnimatedSprite
    /// </summary>
    /// <param name="textureName"></param>
    public AnimatedSprite(string textureName)
      : base(textureName)
    {
    }

    /// <summary>
    /// Creates a new AnimatedSprite with a default animation
    /// </summary>
    /// <param name="textureName">Name of the texture to use for this sprite.</param>
    public AnimatedSprite(string textureName, Animation animation)
      : base(textureName)
    {
      animations.Add("default", animation);
      currentAnimationKey = "default";
      sourceRect =
        animation.Frames[animation.AnimationSequence[animation.CurrentFrame]];
    }

    public AnimatedSprite(SpriteSheetInfo ssi) : base(ssi.TextureName)
    {
    }

    public AnimatedSprite(SpriteSheetInfo ssi, Animation animation)
      : this(ssi.TextureName, animation)
    {
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// Adds an animation to the animated sprite.
    /// </summary>
    /// <param name="key">Animation identifier.</param>
    /// <param name="animation">The Animation object to add.</param>
    public void AddAnimation(string key, Animation animation)
    {
      animations.Add(key, animation);
      if (currentAnimationKey == null)
      {
        currentAnimationKey = key;
        sourceRect =
          animation.Frames[animation.AnimationSequence[animation.CurrentFrame]];
      }
    }

    /// <summary>
    /// Sets the current animation to display.
    /// </summary>
    /// <param name="animationKey">The string identifier for the animation to set.</param>
    public void SetAnimation(string animationKey)
    {
      // This animation is already set
      if (currentAnimationKey.Equals(animationKey))
        return;

      currentAnimationKey = animationKey;
      Animation currentAnimation = animations[currentAnimationKey];
      if (!currentAnimation.IsLooped)
      {
        currentAnimation.IsStarted = true;
        isVisible = true;
        currentAnimation.CurrentFrame = currentAnimation.AnimationSequence[0];
        elapsedTime = 0.0f;
      }
    }

    /// <summary>
    /// Updates the animation, moving between frames each time
    /// animationSpeed seconds have elapsed. This method must be called
    /// in the update method of your game, or in an encapsulating class
    /// if you want the sprite to animate.
    /// </summary>
    /// <param name="gameTime">GameTime object from the XNA framework's Update</param>
    public void UpdateAnimation(GameTime gameTime)
    {
      Animation animation = animations[currentAnimationKey];
      if (animation.IsStarted)
      {
        elapsedTime += (float) gameTime.ElapsedGameTime.TotalSeconds;
        if (elapsedTime >= animation.AnimationSpeed)
        {
          animation.CurrentFrame++;
          if (animation.CurrentFrame >= animation.AnimationSequence.Length)
          {
            animation.IsStarted = animation.IsLooped;
            animation.CurrentFrame = 0;

            if (!animation.IsLooped)
              isVisible = false;
          }
          elapsedTime = 0.0f;
        }

        if (animation.IsStarted)
          sourceRect =
            animation.Frames[animation.AnimationSequence[animation.CurrentFrame]];
      }
    }

    #endregion

    #region Properties

    /// <summary>
    /// Gets the current animation.
    /// </summary>
    public Animation CurrentAnimation
    {
      get { return animations[currentAnimationKey]; }
    }

    #endregion
  }
}