using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace XQUEST.SpriteAnimation
{
  /// <summary>
  /// Holds information about an Animation, such as what frames it consists, 
  /// if it's looped and the animation speed.
  /// </summary>
  public class Animation
  {
    #region Fields

    private List<Rectangle> frames;
    private float animationSpeed;
    private int[] animationSequence;

    private bool isStarted;
    private bool isLooped;

    private int currentFrame;

    #endregion

    #region Constructors

    /// <summary>
    /// Creates a new Animation.
    /// </summary>
    /// <param name="frames">A list of rectangles representing frames in a texture 
    /// that the animation consists of.</param>
    /// <param name="animationSpeed">The delay between each frame during animation, 
    /// in seconds.</param>
    /// <param name="isLooped">If the animation should be loop, or not</param>
    public Animation(List<Rectangle> frames, float animationSpeed, bool isLooped)
      : this(frames, animationSpeed, isLooped, GetDefaultSequence(frames.Count))
    {
    }

    /// <summary>
    /// Creates a new Animation.
    /// </summary>
    /// <param name="frames">A list of rectangles representing frames in a texture that 
    /// the animation consists of.</param>
    /// <param name="isLooped">If the animation should be loop, or not</param>
    /// <param name="animationSpeed">The delay between each frame during animation, 
    /// in seconds.</param>
    /// <param name="animationSequence">A specific sequence the rectangles should be
    /// drawn in. Maybe they should be drawn like 1 2 3 2 1 ?</param>
    public Animation(List<Rectangle> frames, float animationSpeed, bool isLooped,
                     int[] animationSequence)
    {
      this.frames = frames;
      this.isLooped = isLooped;
      this.animationSpeed = animationSpeed;
      this.animationSequence = animationSequence;
      isStarted = true;
    }

    #endregion

    #region Public Methods

    public static int[] GetDefaultSequence(int numFrames)
    {
      int[] sequence = new int[numFrames];
      for (int i = 0; i < numFrames; i++)
      {
        sequence[i] = i;
      }
      return sequence;
    }

    #endregion

    #region Properties

    /// <summary>
    /// Gets or sets the animation sequence.
    /// </summary>
    public int[] AnimationSequence
    {
      get { return animationSequence; }
      set { animationSequence = value; }
    }

    /// <summary>
    /// Gets or sets the animation speed (pause between frames).
    /// </summary>
    public float AnimationSpeed
    {
      get { return animationSpeed; }
      set { animationSpeed = value; }
    }

    /// <summary>
    /// Gets or sets a boolean value indicating whether the animation is started.
    /// </summary>
    public bool IsStarted
    {
      get { return isStarted; }
      set { isStarted = value; }
    }

    /// <summary>
    /// Gets or sets a boolean value indicating whether the animation should loop.
    /// </summary>
    public bool IsLooped
    {
      get { return isLooped; }
      set { isLooped = value; }
    }

    /// <summary>
    /// Gets or sets the list of source rectangles for the individual frames in the 
    /// animation.
    /// </summary>
    public List<Rectangle> Frames
    {
      get { return frames; }
      set { frames = value; }
    }

    /// <summary>
    /// Gets or sets the current frame.
    /// </summary>
    public int CurrentFrame
    {
      get { return currentFrame; }
      set { currentFrame = value; }
    }

    #endregion
  }
}